#! /usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import print_function

# import funkcí z jiného adresáře
import sys
import os.path
import unittest
import scipy
import numpy as np
# import imcut.dcmreaddata

import logging
logger = logging.getLogger(__name__)

path_to_script = os.path.dirname(os.path.abspath(__file__))
sys.path.append(os.path.join(path_to_script, "../src/"))

from nose.plugins.attrib import attr
from nose.tools import raises
from imcut import graph

class DcmReadDataTest(unittest.TestCase):
    # @classmethod
    # def setUpClass(cls):
    #     if sys.version_info.major < 3:
    #         cls.assertCountEqual = cls.assertItemsEqual
    def test_dcmread(self):
        pass



#! /usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import print_function

# import funkcí z jiného adresáře
import sys
import os.path
import unittest
import scipy
import numpy as np

import logging
logger = logging.getLogger(__name__)

path_to_script = os.path.dirname(os.path.abspath(__file__))
sys.path.append(os.path.join(path_to_script, "../src/"))

from nose.plugins.attrib import attr
from imcut import pycut

def fv_function(data, voxelsize, seeds=None, cls=None):
    """
    Creates feature vector for only data or for data from classes
    """

    fv1 = data.reshape(-1,1)

    data2 = scipy.ndimage.filters.gaussian_filter(data, sigma=0.1)
    fv2 = data2.reshape(-1,1)

    fv = np.hstack([fv1, fv2])

    if seeds is not None:
        logger.debug("seeds " + str(seeds))
        print("seeds ", seeds)
        sd = seeds.reshape(-1,1)
        selection = np.in1d(sd, cls)
        fv = fv[selection]
        sd = sd[selection]
        # sd = sd[]
        return fv, sd
    return fv

def box_data(noise_sigma=3):
    # data
    img3d = np.random.rand(32, 64, 64) * noise_sigma
    img3d[4:24, 12:32, 5:25] = img3d[4:24, 12:32, 5:25] + 30

    # seeds
    seeds = np.zeros([32, 64, 64], np.int8)
    seeds[9:12, 13:29, 18:25] = 1
    seeds[9:12, 4:9, 3:32] = 2
    # [mm]  10 x 10 x 10        # voxelsize_mm = [1, 4, 3]
    voxelsize_mm = [5, 5, 5]
    metadata = {'voxelsize_mm': voxelsize_mm}
    return img3d, seeds, voxelsize_mm


class PycutTest(unittest.TestCase):
    def setUp(self):
        self.segparams1 = {
            # 'method':'graphcut',
            'method': 'graphcut',
            'use_boundary_penalties': False,
            'boundary_dilatation_distance': 2,
            'boundary_penalties_weight': 1,
            'modelparams': {
                'type': 'gmmsame',
                "params": {
                    "n_components": 2,
                }
                # 'fv_type': "fv_extern",
                # 'fv_extern': fv_function,
                # 'adaptation': 'original_data',
            }
        }

    @classmethod
    def setUpClass(cls):
        if sys.version_info.major < 3:
            cls.assertCountEqual = cls.assertItemsEqual

    def test_simple_graph_cut(self):
        img, seg, seeds = self.make_data(64, 20)
        gc = pycut.ImageGraphCut(img , segparams=self.segparams1)
        gc.set_seeds(seeds)

        gc.run()
        # import sed3
        # ed = sed3.sed3((gc.segmentation==0).astype(np.double), contour=seg)
        # ed.show()

        err = np.sum(np.abs((gc.segmentation == 0).astype(np.int8) - seg.astype(np.int8)))
        self.assertLess(err, 600)

    def test_simple_graph_cut_show_similarity(self):
        img, seg, seeds = self.make_data(64, 20)
        gc = pycut.ImageGraphCut(img , segparams=self.segparams1)
        gc.set_seeds(seeds)

        gc.run()
        gc.show_similarity(show=False)
        gc.show_model(start=-500, stop=500, show=False)
        # import sed3
        # ed = sed3.sed3((gc.segmentation==0).astype(np.double), contour=seg)
        # ed.show()

        err = np.sum(np.abs((gc.segmentation == 0).astype(np.int8) - seg.astype(np.int8)))
        self.assertLess(err, 600)

    def test_simple_graph_cut_feature_vector_fv001_intensity_and_blur(self):
        # test feature vector with
        segparams = {
            # 'method':'graphcut',
            'method': 'graphcut',
            'use_boundary_penalties': False,
            'boundary_dilatation_distance': 2,
            'boundary_penalties_weight': 1,
            'modelparams': {
                'type': 'gmmsame',
                "params": {
                    "n_components": 2,
                },
            "fv_type": "intensity_and_blur"
                # 'fv_type': "fv_extern",
                # 'fv_extern': fv_function,
                # 'adaptation': 'original_data',
            }
        }
        img, seg, seeds = self.make_data(64, 20)
        gc = pycut.ImageGraphCut(img , segparams=segparams)
        gc.set_seeds(seeds)

        gc.run()
        # outputs looks strange
        # gc.show_similarity(show=True)
        # gc.show_model(start=-500, stop=500, show=True)
        # import sed3
        # ed = sed3.sed3(img, contour=gc.segmentation)
        # ed.show()
        # ed = sed3.sed3((gc.segmentation==0).astype(np.double), contour=seg)
        # ed.show()

        err = np.sum(np.abs((gc.segmentation == 0).astype(np.int8) - seg.astype(np.int8)))
        # TODO check multidimensional feature vector. It looks to be not sufficient.
        # self.assertLess(err, 2000)

    def test_gc_box_overfiting(self):
        data3d, seeds, voxelsize = box_data(noise_sigma=0.5)
        segparams = {
            # 'method':'graphcut',
            'method': 'graphcut',
            'use_boundary_penalties': False,
            'boundary_dilatation_distance': 2,
            'boundary_penalties_weight': 1,
            'modelparams': {
                'type': 'gmmsame',
                "params": {
                    "n_components": 3,
                },
                # 'fv_type': "fv_extern",
                # 'fv_extern': fv_function,
                # 'adaptation': 'original_data',
            }
        }
        gc = pycut.ImageGraphCut(data3d , segparams=segparams, debug_images=False)
        gc.set_seeds(seeds)
        gc.run()

        # import sed3
        # ed = sed3.sed3(data3d, contour=(gc.segmentation==0).astype(np.double) * 3)
        # ed.show()

    def test_simple_graph_cut_overfit_with_low_noise(self):
        img, seg, seeds = self.make_data(64, 20, sigma=20)
        segparams = {
            # 'method':'graphcut',
            'method': 'graphcut',
            'use_boundary_penalties': False,
            'boundary_dilatation_distance': 2,
            'boundary_penalties_weight': 1,
            'modelparams': {
                'type': 'gmmsame',
                "params": {
                    "n_components": 3,
                },
                # 'fv_type': "fv_extern",
                # 'fv_extern': fv_function,
                # 'adaptation': 'original_data',
            }
        }
        gc = pycut.ImageGraphCut(img , segparams=segparams, debug_images=False)
        gc.set_seeds(seeds)

        gc.run()
        # import sed3
        # ed = sed3.sed3(img, contour=(gc.segmentation==0).astype(np.double))
        # ed.show()

        err = np.sum(np.abs((gc.segmentation == 0).astype(np.int8) - seg.astype(np.int8)))
        self.assertLess(err, 600)

    @attr('interactive')
    def test_show_editor(self):
        """
        just run editor to see what is new
        Returns:

        """
        import imcut.seed_editor_qt
        import numpy as np
        from PyQt4.QtGui import QApplication
        app = QApplication(sys.argv)
        data = (np.random.rand(30,31,32) * 100).astype(np.int)
        data[15:40, 13:20, 10:18] += 50
        se = imcut.seed_editor_qt.QTSeedEditor(data)
        se.exec_()
        # self.assertTrue(False)

    # @attr('interactive')
    def test_draw_seed_function(self):
        """
        just run editor to see what is new
        Returns:

        """
        import imcut.seed_editor_qt
        import numpy as np
        from PyQt4.QtGui import QApplication
        app = QApplication(sys.argv)
        data = (np.random.rand(30,31,32) * 100).astype(np.int)
        data[15:40, 13:20, 10:18] += 50
        se = imcut.seed_editor_qt.QTSeedEditor(data)
        se.slice_box.seed_mark = 1 #left mouse button
        se.slice_box.last_position = [1, 3]
        se.slice_box.drawSeeds([10, 5])
        se.slice_box.seed_mark = 2 #left mouse button
        se.slice_box.last_position = [8, 1]
        se.slice_box.drawSeeds([7, 5])
        # se.exec_()

    # @TODO znovu zprovoznit test

    # @unittest.skip("Cekame, az to Tomas opravi")
    def make_data(self, sz=32, offset=0, sigma=80):
        seeds = np.zeros([sz, sz, sz], dtype=np.int8)
        seeds[offset + 12, offset + 9:offset + 14, offset + 10] = 1
        seeds[offset + 20, offset + 18:offset + 21, offset + 12] = 1
        img = np.ones([sz, sz, sz])
        img = img - seeds

        seeds[
            offset + 3:offset + 15, 
            offset + 2:offset + 6, 
            offset + 27:offset + 29] = 2
        img = scipy.ndimage.morphology.distance_transform_edt(img)
        segm = img < 7
        img = (100 * segm + sigma * np.random.random(img.shape)).astype(np.uint8)
        return img, segm, seeds

    def test_remove_repetitive(self):
        import time
        start = time.time()
        nlinks_not_unique = np.random.randint(0, 5, [100000, 3])
        nlinks = pycut.ms_remove_repetitive_link(nlinks_not_unique)
        elapsed = (time.time() - start)
        # print "elapsed ", elapsed

    def test_external_fv(self):
        """
        test external feature vector
        """

        img, seg, seeds = self.make_data(64, 20)
        segparams = {
            # 'method':'graphcut',
            'method': 'graphcut',
            'use_boundary_penalties': False,
            'boundary_dilatation_distance': 2,
            'boundary_penalties_weight': 1,
            'modelparams': {
                'fv_type': "fv_extern",
                'fv_extern': fv_function,
            }
        }
        gc = pycut.ImageGraphCut(img, segparams=segparams)
        gc.set_seeds(seeds)
        gc.run()
        # import sed3
        # ed = sed3.sed3(gc.segmentation==0, contour=seg)
        # ed.show()

        self.assertLess(
            np.sum(
                np.abs(
                    (gc.segmentation == 0).astype(np.int8) - seg.astype(np.int8))
            ),
            600)



    def test_test_fv_function(self):
        img, seg, seeds = self.make_data(64, 20)
        vs = [1,1,1]
        out = fv_function(img, voxelsize=vs)
        print(out.shape)
        out1, out2 = fv_function(img, vs, seeds, [1,2])
        print(np.min(out1), np.max(out1), out1.shape)
        print(np.min(out2), np.max(out2), out2.shape)
        self.assertEqual(out.shape[0], np.prod(img.shape))
        self.assertEqual(out1.shape[0], out2.shape[0])
        self.assertEqual(np.min(out2), 1)
        self.assertEqual(np.max(out2), 2)

    def test_external_fv_with_save(self):
        """
        test external feature vector with save model in the middle of processing
        """

        img, seg, seeds = self.make_data(64, 20)
        segparams = {
            # 'method':'graphcut',
            'method': 'graphcut',
            'use_boundary_penalties': False,
            'boundary_dilatation_distance': 2,
            'boundary_penalties_weight': 1,
            'modelparams': {
                'type': 'gmmsame',
                'fv_type': "fv_extern",
                'fv_extern': fv_function,
                'adaptation': 'original_data',
            }
        }
        gc = pycut.ImageGraphCut(img, segparams=segparams)
        gc.set_seeds(seeds)

        gc.run()
        # import sed3
        # ed = sed3.sed3((gc.segmentation==0).astype(np.double), contour=seg)
        # ed.show()

        err = np.sum(np.abs((gc.segmentation == 0).astype(np.int8) - seg.astype(np.int8)))
        self.assertLess(err, 600)

        mdl_stored_file = "test_model.p"
        gc.save(mdl_stored_file)

        # forget
        gc = None


        img, seg, seeds = self.make_data(56, 18)
        # there is only one change in mdl params
        segparams['modelparams'] = {
            'mdl_stored_file': mdl_stored_file,
            'fv_extern': fv_function
        }
        gc = pycut.ImageGraphCut(img, segparams=segparams)
        gc.set_seeds(seeds)
        gc.run()

        err = np.sum(np.abs((gc.segmentation == 0).astype(np.int8) - seg.astype(np.int8)))
        self.assertLess(err, 600)
        # import sed3
        # sed3.show_slices(img, contour=gc.segmentation==0, slice_step=6)


        # if we change the data there should be more error (assertMore)
        img = (img * 0.2).astype(np.uint8)
        # segparams['modelparams']['adaptation'] = 'original_data'
        # print(np.max(img))
        # print(np.min(img))
        gc = pycut.ImageGraphCut(img, segparams=segparams)
        gc.set_seeds(seeds)
        gc.run()

        m0 = gc.mdl.mdl[1]
        m1 = gc.mdl.mdl[2]
        logger.debug("model parameters")

        # import sed3
        # ed = sed3.sed3((gc.segmentation==0).astype(np.double), contour=seg)
        # ed.show()

        err = np.sum(np.abs((gc.segmentation == 0).astype(np.int8) - seg.astype(np.int8)))
        self.assertGreater(err, 600)
        # self.assertGreater(
        #     np.sum(
        #         np.abs(
        #             (gc.segmentation == 0).astype(np.int8) - seg.astype(np.int8))
        #     ),
        #     600)

        os.remove(mdl_stored_file)

    def test_apriori(self):
        """
        Test apriori segmentation. Make segmentation twice. First is used with gamma=0.2,
        second is used with gamma=0.9
        """

        img, seg, seeds = self.make_data(64, 20)
        apriori = np.zeros([64,64,64])

        apriori[:20,:20,:20] = 1
        segparams1 = { 'apriori_gamma':.1 }
        gc = pycut.ImageGraphCut(img, segparams=segparams1)
        gc.set_seeds(seeds)
        gc.apriori = apriori
        gc.run()
        # import sed3
        # ed = sed3.sed3(img, contour=(gc.segmentation==0))
        # ed.show()

        self.assertLess(
            np.sum(
                np.abs(
                    (gc.segmentation == 0).astype(np.int8) - seg.astype(np.int8))
            ),
            600)



        segparams2 = { 'apriori_gamma':.9 }
        gc = pycut.ImageGraphCut(img, segparams=segparams2)
        gc.set_seeds(seeds)
        gc.apriori = apriori
        gc.run()

        self.assertLess(
            np.sum(
                np.abs(
                    (gc.segmentation == 0).astype(np.int8) - apriori.astype(np.int8))
            ),
            600)

    def test_multiscale_gc_seg(self):
        """
        Test multiscale segmentation
        """

        img, seg, seeds = self.make_data(64, 20)
        segparams = {
            # 'method':'graphcut',
            'method': 'multiscale_graphcut',
            'use_boundary_penalties': False,
            'boundary_dilatation_distance': 2,
            'boundary_penalties_weight': 1,
            'block_size': 8,
            'tile_zoom_constant': 1
        }
        gc = pycut.ImageGraphCut(img, segparams=segparams)
        gc.set_seeds(seeds)
        gc.run()
        # import sed3
        # ed = sed3.sed3(gc.segmentation==0, contour=seg)
        # ed.show()

        self.assertLess(
            np.sum(
                np.abs(
                    (gc.segmentation == 0).astype(np.int8) - seg.astype(np.int8))
            ),
            600)

    def test_fast_multiscale_gc_seg(self):
        """
        Test multiscale segmentation
        """

        img, seg, seeds = self.make_data(64, 20)
        segparams = {
            # 'method':'graphcut',
            'method': 'multiscale_graphcut',
            'use_boundary_penalties': False,
            'boundary_dilatation_distance': 2,
            'boundary_penalties_weight': 1,
            'block_size': 8,
            'tile_zoom_constant': 1
        }
        gc = pycut.ImageGraphCut(img, segparams=segparams)
        gc.set_seeds(seeds)
        gc.run()
        # import sed3
        # ed = sed3.sed3(gc.segmentation==0, contour=seg)
        # ed.show()

        self.assertLess(
            np.sum(
                np.abs(
                    (gc.segmentation == 0).astype(np.int8) - seg.astype(np.int8))
            ),
            600)

    # @unittest.skip("Cekame, az to Mire opravi")
    def test_ms_seg_compared_with_different_resolution(self):
        """
        Test multiscale segmentation
        """

        img, seg, seeds = self.make_data(64, 20)
        segparams = {
                # 'method':'graphcut',
                'method': 'multiscale_graphcut_hi2lo',
                'use_boundary_penalties': False,
                'boundary_dilatation_distance': 2,
                'boundary_penalties_weight': 1,
                'block_size': 8,
                'tile_zoom_constant': 1
                }
        gc = pycut.ImageGraphCut(img, segparams=segparams)
        gc.set_seeds(seeds)
        gc.run()
        # import sed3
        # ed = sed3.sed3(gc.segmentation==0, contour=seg)
        # ed.show()

        self.assertLess(
                np.sum(
                    np.abs(
                        (gc.segmentation == 0).astype(np.int8) - seg.astype(np.int8))
                    ),
                600)


# different resolution
        # sz = [128,128,128]
        sz = [70,70,70]
        sz = [90,90,90]
        sz = [100,100,100]
        sz = [200,200,200]
        sz1 = 70
        sz = [sz1, sz1, sz1]
        img2 = pycut.zoom_to_shape(img, sz, np.uint8)
        seg2 = pycut.zoom_to_shape(seg, sz, np.uint8)
        seeds2 = pycut.zoom_to_shape(seeds, sz, np.int8)

        segparams['tile_zoom_constant'] = 0.8
        gc = pycut.ImageGraphCut(img2, segparams=segparams)
        gc.set_seeds(seeds2)
        gc.run()
        # import sed3
        # ed = sed3.sed3(gc.segmentation==0, contour=seg2)
        # ed.show()
        

    def test_segmentation(self):
        img, seg, seeds = self.make_data()
        gc = pycut.ImageGraphCut(img)
        gc.set_seeds(seeds)
        gc.run()
        # import sed3
        # ed = sed3.sed3(gc.segmentation==0, contour=seg)
        # ed.show()
        self.assertLess(
                np.sum(
                    np.abs(
                        (gc.segmentation == 0).astype(np.int8) - 
                        seg.astype(np.int8))
                    )
                , 30)
        
    def test_multiscale_indexes(self):
        # there must be some data
        img = np.zeros([32, 32, 32], dtype=np.int16)
        gc = pycut.ImageGraphCut(img)

        # mask = np.zeros([1, 4, 4], dtype=np.int16)
        # mask[0,1:3,2:] = 1
        mask = np.zeros([1, 3, 3], dtype=np.int16)
        mask[0, 1:, 1] = 1
        orig_shape = [2, 6, 6]
        # zoom = 2

        inds = gc._ImageGraphCut__multiscale_indexes(mask, orig_shape)

        expected_result = [[[0, 0,  1,  1, 2, 2],
                            [0, 0,  1,  1, 2, 2],
                            [3, 3,  7,  8, 4, 4],
                            [3, 3,  9, 10, 4, 4],
                            [5, 5, 11, 12, 6, 6],
                            [5, 5, 13, 14, 6, 6]],
                           [[0, 0,  1,  1, 2, 2],
                            [0, 0,  1,  1, 2, 2],
                            [3, 3, 15, 16, 4, 4],
                            [3, 3, 17, 18, 4, 4],
                            [5, 5, 19, 20, 6, 6],
                            [5, 5, 21, 22, 6, 6]]]

        self.assertCountEqual(inds.reshape(-1),
                              np.array(expected_result).reshape(-1))

    def test_ordered_values_by_indexes(self):
        """
        test of pycut.__ordered_values_by_indexes
        """

        # there must be some data
        img = np.zeros([32, 32, 32], dtype=np.int16)
        data = np.array([
            [0, 1, 1],
            [0, 2, 2],
            [0, 2, 2]
        ])

        inds = np.array([
            [0, 1, 2],
            [3, 4, 4],
            [5, 4, 4]
        ])
        gc = pycut.ImageGraphCut(img)
        vals = gc._ImageGraphCut__ordered_values_by_indexes(data, inds)
        expected = np.array([0, 1, 1, 0, 2, 0])
        self.assertCountEqual(vals, expected)

    def test_ordered_values_by_indexes_with_different_values(self):
        """
        test of pycut.__ordered_values_by_indexes
        in input data are non-consistent data
        Function should take maximal value
        """

        # there must be some data
        img = np.zeros([32, 32, 32], dtype=np.int16)
        data = np.array([
            [0, 1, 1],
            [0, 2, 2],
            [0, 3, 2]
        ])

        inds = np.array([
            [0, 1, 2],
            [3, 4, 4],
            [5, 4, 4]
        ])
        gc = pycut.ImageGraphCut(img)
        vals = gc._ImageGraphCut__ordered_values_by_indexes(data, inds)
        expected = np.array([0, 1, 1, 0, 3, 0])
        self.assertCountEqual(vals, expected)

    # @unittest.skip("This test is almost done")
    def test_msgc_lo2hi(self):
        """
        Test multiscale segmentation
        """

        img, seg, seeds = self.make_data(64, 20)
        segparams = {
            # 'method':'graphcut',
            'method': 'multiscale_graphcut_lo2hi',
            # 'method': 'multiscale_graphcut_hi2lo',
            'use_boundary_penalties': False,
            'boundary_dilatation_distance': 2,
            'boundary_penalties_weight': 1,
            'block_size': 8,
            'tile_zoom_constant': 1
        }
        gc = pycut.ImageGraphCut(img, segparams=segparams, keep_graph_properties=True)
        gc.set_seeds(seeds)
        gc.run()

        edseeds = seeds
        # gc.interactive_inspect_node()

        # node_unariesalt, node_neighboor_edges_and_weights, node_neighboor_seeds = gc.inspect_node(edseeds)

        self.assertLess(
            np.sum(
                np.abs(
                    (gc.segmentation == 0).astype(np.int8) - seg.astype(np.int8))
            ),
            600)

    def test_node_inspection_on_msgc_lo2hi(self):
        """
        Test multiscale segmentation
        """

        img, seg, seeds = self.make_data(64, 20)
        segparams = {
            # 'method':'graphcut',
            'method': 'multiscale_graphcut_lo2hi',
            # 'method': 'multiscale_graphcut_hi2lo',
            'use_boundary_penalties': False,
            'boundary_dilatation_distance': 2,
            'boundary_penalties_weight': 1,
            'block_size': 8,
            'tile_zoom_constant': 1
        }
        gc = pycut.ImageGraphCut(img, segparams=segparams, keep_graph_properties=True)
        gc.set_seeds(seeds)
        gc.run()

        self.assertLess(
            np.sum(
                np.abs(
                    (gc.segmentation == 0).astype(np.int8) - seg.astype(np.int8))
            ),
            600)
        # gc.interactive_inspect_node()

        node_unariesalt, node_neighboor_edges_and_weights, node_neighboor_seeds = gc.inspect_node(seeds)
        ii, ij, ik = np.nonzero(node_neighboor_seeds)
        #
        self.assertLessEqual(np.max(ii) - np.min(ii), 24, msg="Neighbor nodes position variance should be maximum 3 * block size ")
        self.assertLessEqual(np.max(ij) - np.min(ij), 24, msg="Neighbor nodes position variance should be maximum 3 * block size ")
        self.assertLessEqual(np.max(ik) - np.min(ik), 24, msg="Neighbor nodes position variance should be maximum 3 * block size ")

    def test_msgc_lo2hi_crazy_non_block_sized_images_65(self):
        """
        Test multiscale segmentation
        """

        img, seg, seeds = self.make_data(65, 20)
        segparams = {
            # 'method':'graphcut',
            'method': 'multiscale_graphcut_lo2hi',
            # 'method': 'multiscale_graphcut_hi2lo',
            'use_boundary_penalties': False,
            'boundary_dilatation_distance': 2,
            'boundary_penalties_weight': 1,
            'block_size': 8,
            'tile_zoom_constant': 1
        }
        gc = pycut.ImageGraphCut(img, segparams=segparams, keep_graph_properties=True)
        gc.set_seeds(seeds)
        gc.run()

        edseeds = seeds
        # gc.interactive_inspect_node()

        # node_unariesalt, node_neighboor_edges_and_weights, node_neighboor_seeds = gc.inspect_node(edseeds)

        self.assertLess(
            np.sum(
                np.abs(
                    (gc.segmentation == 0).astype(np.int8) - seg.astype(np.int8))
            ),
            600)

    def test_msgc_lo2hi_crazy_non_block_sized_images_68(self):
        """
        Test multiscale segmentation
        """

        img, seg, seeds = self.make_data(68, 20)
        segparams = {
            # 'method':'graphcut',
            'method': 'multiscale_graphcut_lo2hi',
            # 'method': 'multiscale_graphcut_hi2lo',
            'use_boundary_penalties': False,
            'boundary_dilatation_distance': 2,
            'boundary_penalties_weight': 1,
            'block_size': 8,
            'tile_zoom_constant': 1
        }
        gc = pycut.ImageGraphCut(img, segparams=segparams, keep_graph_properties=True)
        gc.set_seeds(seeds)
        gc.run()

        edseeds = seeds
        # gc.interactive_inspect_node()

        # node_unariesalt, node_neighboor_edges_and_weights, node_neighboor_seeds = gc.inspect_node(edseeds)

        self.assertLess(
            np.sum(
                np.abs(
                    (gc.segmentation == 0).astype(np.int8) - seg.astype(np.int8))
            ),
            600)

if __name__ == "__main__":
    unittest.main()



#! /usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import print_function

# import funkcí z jiného adresáře
import sys
import os.path
import unittest
import scipy
import numpy as np

import logging
logger = logging.getLogger(__name__)

path_to_script = os.path.dirname(os.path.abspath(__file__))
sys.path.append(os.path.join(path_to_script, "../src/"))

from nose.plugins.attrib import attr
from nose.tools import raises
from imcut import graph

orig_sr_tab = {
    2: np.array([(0,2), (0,1), (1,3), (2,3)]),
    3: np.array([(0,3,6), (0,1,2), (2,5,8), (6,7,8)]),
    4: np.array([(0,4,8,12), (0,1,2,3), (3,7,11,15), (12,13,14,15)]),
}

class GraphTest(unittest.TestCase):
    # @classmethod
    # def setUpClass(cls):
    #     if sys.version_info.major < 3:
    #         cls.assertCountEqual = cls.assertItemsEqual
    def test_graph_2d_implementation(self):

        data = np.array([[0,0,0,0,1,1,0],
                         [0,1,1,0,1,0,1],
                         [0,1,1,0,1,0,0],
                         [1,1,1,0,0,0,0],
                         [0,0,0,0,0,0,1]])
        g = graph.Graph(data, (0.1, 0.12), grid_function="2d", nsplit=7)
        # g = graph.Graph(data, (0.1, 0.12), grid_function="nd", nsplit=5)
        g.run(base_grid_vtk_fn="base_grid.vtk", final_grid_vtk_fn="final_grid.vtk")
        # 2d edge number diff= nsplit:(g.lastedge - g.edmax_debug)
        #
        # 2: 6, 3:15, 4:24
        g

    def test_graph_3d_implementation(self):

        data = np.array([[0,0,0,0,1,1,0],
                         [0,1,1,0,1,0,1],
                         [0,1,1,0,1,0,0],
                         [1,1,1,0,0,0,0],
                         [0,0,0,0,0,0,1]])
        # g = graph.Graph(data, (0.1, 0.12), grid_function="2d", nsplit=3)
        # 3d 2:0,
        # diff 3d 2:6, 3:15, 4:24, 5:33 6:42 7:51
        g = graph.Graph(data, (0.1, 0.12), grid_function="nd", nsplit=7)

        g.run(base_grid_vtk_fn="base_grid.vtk", final_grid_vtk_fn="final_grid.vtk")
        g.lastnode

    @unittest.skip("waiting for fix")
    def test_graph_3d_two_slices(self):

        data = np.array(
            [
                [[0,0,0,0,1,1,0],
                 [0,1,1,0,1,0,1],
                 [0,1,1,0,1,0,0],
                 [1,1,1,0,0,0,0],
                 [0,0,0,0,0,0,1]],
                [[0,0,0,0,0,0,0],
                 [0,0,0,0,0,0,0],
                 [0,1,1,0,0,0,0],
                 [0,1,1,0,0,0,0],
                 [0,0,0,0,0,0,1]],
            ]
        )
        g = graph.Graph(data, (0.1, 0.12, 0.05))
        g.run()

    # @unittest.skip("waiting for fix")
    def test_graph_3d(self):

        data = np.array(
            [
                [[0,0,0,0,0],
                 [0,1,1,0,1],
                 [1,1,1,0,0],
                 [0,0,0,0,1]],
                [[0,0,0,0,0],
                 [0,0,0,0,0],
                 [0,1,1,0,0],
                 [0,0,0,0,1]],
                [[0,0,0,0,0],
                 [0,0,0,0,0],
                 [0,1,1,0,0],
                 [0,0,0,0,1]],
            ]
        )
        g = graph.Graph(data, (0.1, 0.12, 0.05), grid_function="nd", nsplit=6)
        # 2:-21, 3:10, 4:57 5:120 6:199
        31, 47, 63, 79
        16, 16, 16
        g.run(base_grid_vtk_fn="base_grid.vtk", final_grid_vtk_fn="final_grid.vtk")
        g.lastnode
        # g.run()

    def test_just_two_blocks(self):

        data = np.array(
            [
                [[0, 1]]
            ]
        )
        g = graph.Graph(data, (0.1, 0.2, 0.05), grid_function="nd", nsplit=2)
        g.run(base_grid_vtk_fn="base_grid.vtk", final_grid_vtk_fn="final_grid.vtk")
        g
        # self.assertEqual(g.edges.shape[0], 33)
        # self.assertEqual(g.edges.shape[1], 2)
        # self.assertEqual(g.nodes.shape[0], 15)

    def test_into_edges_reconnetcion_on_small_graph_3d(self):

        data = np.array(
            [
                [[0,0,],
                 [0,0,]],
                [[0,0,],
                 [0,1,]],
            ]
        )
        g = graph.Graph(data, (0.1, 0.2, 0.05), grid_function="nd", nsplit=2)
        g.run(base_grid_vtk_fn="base_grid.vtk", final_grid_vtk_fn="final_grid.vtk")
        g.edges
        self.assertEqual(g.edges.shape[0], 33)
        self.assertEqual(g.edges.shape[1], 2)
        self.assertEqual(g.nodes.shape[0], 15)

    def test_from_edges_reconnetcion_on_small_graph_3d(self):

        data = np.array(
            [
                [[1,0,],
                 [0,0,]],
                [[0,0,],
                 [0,0,]],
            ]
        )
        g = graph.Graph(data, (0.1, 0.2, 0.05), grid_function="nd", nsplit=2)
        g.run(base_grid_vtk_fn="base_grid.vtk", final_grid_vtk_fn="final_grid.vtk")
        self.assertEqual(g.edges.shape[0], 33)
        self.assertEqual(g.edges.shape[1], 2)
        self.assertEqual(g.nodes.shape[0], 15)

    def test_msindex_on_small_graph_3d(self):

        data = np.array(
            [
                [[1,0,],
                 [0,0,]],
                [[0,0,],
                 [0,0,]],
            ]
        )
        g = graph.Graph(data, (0.1, 0.2, 0.05), grid_function="nd", nsplit=2, compute_msindex=True)
        g.run(base_grid_vtk_fn="base_grid.vtk", final_grid_vtk_fn="final_grid.vtk")
        self.assertEqual(g.edges.shape[0], 33)
        self.assertEqual(g.edges.shape[1], 2)
        self.assertEqual(g.nodes.shape[0], 15)

        # msindex with low resolution should be in the beginning of data
        un, counts = np.unique(g.msinds, return_counts=True)
        # on this example first 7 node labeles should be used multiple timse
        self.assertTrue((counts[:7] > 1).all(),
                        msg="on this example first 7 node labeles should be used multiple times")

    def test_into_maximum_edge_number(self):

        # data = np.random.random([50, 50, 8])
        data = np.random.random([5, 3, 3])
        data = data < 0.08
        # data = np.random.random_integers(0,1, [10,10,10])
        g = graph.Graph(data, (0.1, 0.2, 0.05), grid_function="nd", nsplit=6)
        g.run(base_grid_vtk_fn="base_grid.vtk", final_grid_vtk_fn="final_grid.vtk")
        unused_edges = g.edmax - g.lastedge
        unused_nodes = g.ndmax - g.lastnode
        edge_efectivity = 1 - float(unused_edges) / g.edmax
        node_efectivity = 1 - float(unused_nodes) / g.ndmax
        logger.info("Memory node efficiency: {} ({})".format(node_efectivity, g.ndmax))
        logger.info("Memory edge efficiency: {} ({})".format(edge_efectivity, g.edmax))

        g


    def test_from_edges_reconnetcion_onsmall_graph_3d_higres_neighborhood(self):

        data = np.array(
            [
                [[1,1,],
                 [1,0,]],
                [[1,0,],
                 [0,0,]],
            ]
        )
        g = graph.Graph(data, (0.1, 0.2, 0.05), grid_function="nd", nsplit=2, compute_msindex=True)
        g.run(base_grid_vtk_fn="base_grid.vtk", final_grid_vtk_fn="final_grid.vtk")
        # self.assertEqual(g.edges.shape[0], 33)
        # self.assertEqual(g.edges.shape[1], 2)
        # self.assertEqual(g.nodes.shape[0], 15)

    def test_into_edges_reconnetcion_onsmall_graph_3d_higres_neighborhood(self):

        data = np.array(
            [
                [[0,0,],
                 [0,1,]],
                [[0,1,],
                 [1,1,]],
            ]
        )
        g = graph.Graph(data, (0.1, 0.2, 0.05), grid_function="nd", nsplit=2)
        g.run(base_grid_vtk_fn="base_grid.vtk", final_grid_vtk_fn="final_grid.vtk")
        # self.assertEqual(g.edges.shape[0], 33)
        # self.assertEqual(g.edges.shape[1], 2)
        # self.assertEqual(g.nodes.shape[0], 15)

    def _test_automatic_ms_indexes_2d_same_as_orig(self, size):
        shape = [size, size]
        srt = graph.SRTab()
        subtab = srt.get_sr_subtab(shape)

        err = np.sum(np.abs(subtab - orig_sr_tab[size]))
        self.assertEqual(err, 0)

    def test_automatic_ms_indexes_2d_same_as_orig_2(self):
        size = 2
        self._test_automatic_ms_indexes_2d_same_as_orig(size)

    def test_automatic_ms_indexes_2d_same_as_orig_3(self):
        size = 3
        self._test_automatic_ms_indexes_2d_same_as_orig(size)

    def test_automatic_ms_indexes_2d_same_as_orig_4(self):
        size = 4
        self._test_automatic_ms_indexes_2d_same_as_orig(size)


    def test_automatic_ms_indexes_3d(self):
        shape = [3, 3, 3]
        srt = graph.SRTab()
        subtab = srt.get_sr_subtab(shape)
        subtab

        # err = np.sum(np.abs(subtab - orig_sr_tab[size]))
        # self.assertEqual(err, 0)

    def test_gen_base_graph_2d(self):
        shape = [2, 3]
        voxelsize = [1., .6]
        # srt = graph.Graph(shape)

        nodes1, edges1, edg_dir1 = graph.gen_grid_2d(shape, voxelsize)
        nodes2, edges2, edg_dir2 = graph.gen_grid_nd(shape, voxelsize)

        graph.write_grid_to_vtk("grid1.vtk", nodes1, edges1)
        graph.write_grid_to_vtk("grid2.vtk", nodes2, edges2)
        nodes1


    def test_multiscale_index_set_lowres(self):
        shape = [2, 3]
        block_size = 2
        msi = graph.MultiscaleIndex(shape, block_size=block_size)
        # set first block
        msi.set_block_lowres(0, 5)
        self.assertEqual(msi.msinds[0, 0], 5)
        # set last block
        msi.set_block_lowres(np.prod(shape) - 1, 3)
        self.assertEqual(msi.msinds[-1, -1], 3)


    def test_multiscale_index_set_higres(self):
        shape = [2, 3]
        block_size = 2
        msi = graph.MultiscaleIndex(shape, block_size=block_size)
        # set last block
        msi.set_block_higres(np.prod(shape) - 1, [11, 12, 13, 14])
        self.assertEqual(msi.msinds[-1, -1], 14)


    @raises(ValueError)
    def test_raise_exception_on_different_input_shapes(self):

        data = np.array(
            [
                [[1,1,],
                 [1,0,]],
                [[1,0,],
                 [0,0,]],
            ]
        )
        g = graph.Graph(data, (0.1, 0.2), grid_function="nd", nsplit=2)
        # g.run(base_grid_vtk_fn="base_grid.vtk", final_grid_vtk_fn="final_grid.vtk")
        # self.assertEqual(g.edges.shape[0], 33)
        # self.assertEqual(g.edges.shape[1], 2)
        # self.assertEqual(g.nodes.shape[0], 15)

if __name__ == "__main__":
    unittest.main()



#! /usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import print_function

# import funkcí z jiného adresáře
import sys
import os.path
import unittest
import scipy
import numpy as np
import imcut.image_manipulation as imma

import logging
logger = logging.getLogger(__name__)

path_to_script = os.path.dirname(os.path.abspath(__file__))
sys.path.append(os.path.join(path_to_script, "../src/"))

from nose.plugins.attrib import attr
from nose.tools import raises
from imcut import graph

class ImageManipulationTest(unittest.TestCase):
    # @classmethod
    # def setUpClass(cls):
    #     if sys.version_info.major < 3:
    #         cls.assertCountEqual = cls.assertItemsEqual
    def test_resize_to_shape_no_new_unique_values(self):
        data = np.zeros([10, 15, 12])
        value1 = 1
        value2 = 2
        data[:5, :7, :6] = value1
        data[-5:, :7, :6] = value2

        expected_shape = [15, 15, 15]
        resized = imma.resize_to_shape(data, expected_shape)
        unique = np.unique(resized)

        self.assertEqual(resized.shape[0], expected_shape[0])
        self.assertEqual(resized.shape[1], expected_shape[1])
        self.assertEqual(resized.shape[2], expected_shape[2])
        self.assertEqual(resized[1, 1, 1], value1)
        self.assertEqual(resized[-2, 1, 1], value2)
        self.assertEqual(len(unique), 3)
        self.assertEqual(unique[0], 0)
        self.assertEqual(unique[1], 1)
        self.assertEqual(unique[2], 2)

    def test_resize_to_shape_wiht_zoom_no_new_unique_values(self):

        data = np.zeros([10, 15, 12])
        value1 = 1
        value2 = 2
        data[:5, :7, :6] = value1
        data[-5:, :7, :6] = value2

        expected_shape = [15, 15, 15]
        zoom = data.shape / np.array(expected_shape, dtype=float)
        resized = imma.resize_to_shape_with_zoom(data, expected_shape, zoom=zoom)
        unique = np.unique(resized)

        self.assertEqual(resized.shape[0], expected_shape[0])
        self.assertEqual(resized.shape[1], expected_shape[1])
        self.assertEqual(resized.shape[2], expected_shape[2])
        self.assertEqual(resized[1, 1, 1], value1)
        self.assertEqual(resized[-2, 1, 1], value2)
        self.assertEqual(len(unique), 3)
        self.assertEqual(unique[0], 0)
        self.assertEqual(unique[1], 1)
        self.assertEqual(unique[2], 2)

    def test_get_priority_objects(self):
        shape = [10, 15, 12]
        data = np.zeros(shape)
        value1 = 1
        value2 = 2
        data[:5, :7, :6] = value1
        data[-5:, :7, :6] = value2

        seeds = np.zeros(shape)
        seeds[9,3:6, 3] = 1

        selected = imma.select_objects_by_seeds(data, seeds)
        unique = np.unique(selected)
        #
        self.assertEqual(selected.shape[0], shape[0])
        self.assertEqual(selected.shape[1], shape[1])
        self.assertEqual(selected.shape[2], shape[2])
        self.assertEqual(selected[1, 1, 1], 1)
        self.assertEqual(selected[-2, 1, 1], 0)
        self.assertEqual(len(unique), 2)
        self.assertGreater(np.count_nonzero(data), np.count_nonzero(selected))

    def test_crop_and_uncrop(self):
        shape = [10, 10, 5]
        img_in = np.random.random(shape)

        crinfo = [[2, 8], [3, 9], [2, 5]]

        img_cropped = imma.crop(img_in, crinfo)

        img_uncropped = imma.uncrop(img_cropped, crinfo, shape)

        self.assertTrue(img_uncropped[4, 4, 3] == img_in[4, 4, 3])

    def test_multiple_crop_and_uncrop(self):
        """
        test combination of multiple crop
        """

        shape = [10, 10, 5]
        img_in = np.random.random(shape)

        crinfo1 = [[2, 8], [3, 9], [2, 5]]
        crinfo2 = [[2, 5], [1, 4], [1, 2]]

        img_cropped = imma.crop(img_in, crinfo1)
        img_cropped = imma.crop(img_cropped, crinfo2)

        crinfo_combined = imma.combinecrinfo(crinfo1, crinfo2)

        img_uncropped = imma.uncrop(img_cropped, crinfo_combined, shape)

        self.assertTrue(img_uncropped[4, 4, 3] == img_in[4, 4, 3])
        self.assertEquals(img_in.shape, img_uncropped.shape)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
SeedEditor for organ segmentation

Example:

$ seed_editor_qp.py -f head.mat
"""

# import unittest
from optparse import OptionParser
from scipy.io import loadmat
import numpy as np
import sys
from scipy.spatial import Delaunay


import PyQt4
from PyQt4.QtCore import Qt, QSize, SIGNAL
try:
    from PyQt4.QtCore import QString
except ImportError:
    # we are using Python3 so QString is not defined
    QString = type("")
from PyQt4.QtGui import QImage, QDialog,\
    QApplication, QSlider, QPushButton,\
    QLabel, QPixmap, QPainter, qRgba,\
    QComboBox, QIcon, QStatusBar,\
    QHBoxLayout, QVBoxLayout, QFrame, QSizePolicy

import logging
logger = logging.getLogger(__name__)

from seededitorqt.seed_editor_qt import QTSeedEditor, main

import math
# BGRA order
# GRAY_COLORTABLE = np.array([[ii, ii, ii, 255] for ii in range(256)],
#                            dtype=np.uint8)
#
# SEEDS_COLORTABLE = np.array([[
#                                  (15 + ii * 41) % 256,
#                                  (47 + ii * 117) % 256,
#                                  (11 + ii * -31) % 256,
#                                  220
#                              ] for ii in range(256)], dtype=np.uint8)
# SEEDS_COLORTABLE[:4] = np.array([[0, 255, 0, 220],
#                              [64, 0, 255, 220],
#                              [0, 200, 128, 220],
#                              [64, 128, 200, 220]], dtype=np.uint8)
#
#
# # In future something like this...
# # CONTOURS_COLORS = np.array([[
# #                                  (15 + ii * 41) % 256,
# #                                  (47 + ii * 117) % 256,
# #                                  (11 + ii * -31) % 256
# #                             ] for ii in range(256)],
# #                             dtype=np.uint8)
# CONTOURS_COLORS = {
#     1: [64, 255, 0],
#     2: [255, 0, 64],
#     3: [0, 64, 255],
#     4: [255, 64, 0],
#     5: [64, 0, 255],
#     6: [0, 255, 64],
#     7: [0, 128, 192],
#     8: [128, 0, 192],
#     9: [128, 192, 0],
#     10: [0, 192, 128],
#     11: [192, 128, 0],
#     12: [192, 0, 128],
#     13: [128, 0, 0],
#     14: [0, 128, 0],
#     15: [0, 0, 128],
#     16: [64, 128, 128],
#     17: [128, 128, 64],
#     18: [128, 64, 128],
#     19: [128, 255, 0],
#     20: [128, 255, 128],
#     21: [128, 128, 255],
#     22: [128, 255, 128],
#     23: [64, 255, 128],
#     24: [0, 255, 128],
#     25: [128, 255, 255],
#     26: [64, 0, 0],
#     27: [0, 0, 64],
#     28: [0, 64, 64],
#     29: [64, 128, 0],
#     30: [192, 128, 64],
#     31: [64, 0, 128],
#     32: [128, 128, 64],
#     33: [0, 128, 64],
#     34: [128, 0, 64],
#     35: [64, 64, 0],
#     36: [0, 64, 0],
#     37: [0, 64, 64],
#     38: [128, 0, 0],
#     39: [128, 255, 0],
#     40: [0, 0, 128],
#     41: [0, 128, 128],
#     42: [64, 128, 0],
#     43: [64, 128, 0],
#     44: [128, 0, 128],
#     45: [128, 64, 128],
#     46: [128, 128, 64],
#     47: [128, 64, 128],
#     48: [64, 64, 128],
#     49: [0, 64, 128],
#     50: [128, 64, 64],
#     51: [255, 64, 64],
#     52: [64, 64, 255],
#     53: [64, 255, 255],
#     54: [255, 128, 64],
#     55: [255, 128, 255],
#     56: [255, 64, 128],
#     57: [128, 128, 255],
#     58: [64, 128, 255],
#     59: [128, 64, 255],
#     60: [255, 255, 64],
#     61: [64, 255, 64],
#     62: [64, 255, 255],
#     63: [128, 64, 64],
#     64: [128, 64, 64],
#     65: [64, 64, 128],
#     66: [64, 128, 128],
#     67: [255, 128, 64],
#     68: [255, 128, 64],
#     69: [128, 64, 128],
#     70: [128, 255, 128],
#     71: [128, 128, 255],
#     72: [128, 255, 128],
#     73: [255, 255, 128],
#     74: [64, 255, 128],
#     75: [128, 255, 255],
#     76: [64, 255, 255],
#     77: [255, 255, 64],
#     78: [255, 64, 64],
#     79: [64, 128, 255],
#     80: [64, 128, 64],
#     81: [64, 255, 128],
#     82: [128, 128, 64],
#     83: [255, 128, 64],
#     84: [128, 255, 64],
#     85: [64, 64, 255],
#     86: [255, 64, 255],
#     87: [255, 64, 64],
#     88: [128, 255, 255],
#     89: [128, 255, 64],
#     90: [255, 255, 128],
#     91: [255, 128, 128],
#     92: [64, 128, 255],
#     93: [64, 128, 255],
#     94: [128, 255, 128],
#     95: [128, 64, 128],
#     96: [128, 128, 64],
#     97: [128, 64, 128],
#     98: [64, 64, 128],
#     99: [255, 64, 128],
#     100: [128, 64, 64],
# }
#
# CONTOURS_COLORTABLE = np.zeros((256,4), dtype=np.uint8)
# CONTOURS_COLORTABLE[:,:3] = 255
# CONTOURLINES_COLORTABLE = np.zeros((256,2,4), dtype=np.uint8)
# CONTOURLINES_COLORTABLE[:,:,:3] = 255
#
# for ii, jj in CONTOURS_COLORS.items():
#     key = ii - 1
#     CONTOURS_COLORTABLE[key,:3] = jj
#     CONTOURS_COLORTABLE[key,3] = 64
#     CONTOURLINES_COLORTABLE[key,0,:3] = jj
#     CONTOURLINES_COLORTABLE[key,0,3] = 16
#     CONTOURLINES_COLORTABLE[key,1,:3] = jj
#     CONTOURLINES_COLORTABLE[key,1,3] = 255
#
# VIEW_TABLE = {'axial': (2,1,0),
#               'sagittal': (1,0,2),
#               'coronal': (2,0,1)}
# DRAW_MASK = [
#     (np.array([[1]], dtype=np.int8), 'small pen'),
#     (np.array([[0, 1, 1, 1, 0],
#                [1, 1, 1, 1, 1],
#                [1, 1, 1, 1, 1],
#                [1, 1, 1, 1, 1],
#                [0, 1, 1, 1, 0]], dtype=np.int8), 'middle pen'),
#     (np.array([[0, 0, 0, 0, 1, 1, 1, 0, 0, 0, 0],
#                [0, 0, 1, 1, 1, 1, 1, 1, 1, 0, 0],
#                [0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0],
#                [0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0],
#                [1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1],
#                [1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1],
#                [1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1],
#                [0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0],
#                [0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0],
#                [0, 0, 1, 1, 1, 1, 1, 1, 1, 0, 0],
#                [0, 0, 0, 0, 1, 1, 1, 0, 0, 0, 0]], dtype=np.int8), 'large pen'),
#     ]
#
# BOX_BUTTONS_SEED = {
#     Qt.LeftButton: 1,
#     Qt.RightButton: 2,
#     }
#
# BOX_BUTTONS_SEED_SHIFT_OFFSET = 2
#
# BOX_BUTTONS_DRAW = {
#     Qt.LeftButton: 1,
#     Qt.RightButton: 0,
#     }
#
# NEI_TAB = [[-1, -1], [0, -1], [1, -1],
#            [-1, 0], [1, 0],
#            [-1, 1], [0, 1], [1, 1]]
#
#
# def erase_reg(arr, p, val=0):
#     from scipy.ndimage.measurements import label
#
#     labs, num = label(arr)
#     aval = labs[p]
#     idxs = np.where(labs == aval)
#     arr[idxs] = val
#
# class SliceBox(QLabel):
#     """
#     Widget for marking reagions of interest in DICOM slices.
#     """
#
#     def __init__(self, sliceSize, grid,
#                  mode='seeds'):
#         """
#         Initialize SliceBox.
#
#         Parameters
#         ----------
#         sliceSize : tuple of int
#             Size of slice matrix.
#         grid : tuple of float
#             Pixel size:
#             imageSize = (grid1 * sliceSize1, grid2 * sliceSize2)
#         mode : str
#             Editor mode.
#         """
#
#         QLabel.__init__(self)
#
#         self.drawing = False
#         self.modified = False
#         self.seed_mark = None
#         self.last_position = None
#         self.imagesize = QSize(int(sliceSize[0] * grid[0]),
#                                int(sliceSize[1] * grid[1]))
#         self.grid = grid
#         self.slice_size = sliceSize
#         self.ctslice_rgba = None
#         self.cw = {'c': 1.0, 'w': 1.0}
#
#         self.seeds = None
#         self.contours = None
#         self.contours_old = None
#         self.mask_points = None
#         self.erase_region_button = None
#         self.erase_fun = None
#         self.erase_mode = 'inside'
#         self.contour_mode = 'fill'
#         self.scroll_fun = None
#
#         if mode == 'draw':
#             self.seeds_colortable = CONTOURS_COLORTABLE
#             self.box_buttons = BOX_BUTTONS_DRAW
#             self.mode_draw = True
#
#         else:
#             self.seeds_colortable = SEEDS_COLORTABLE
#             self.box_buttons = BOX_BUTTONS_SEED
#             self.mode_draw = False
#
#         self.image = QImage(self.imagesize, QImage.Format_RGB32)
#         self.setPixmap(QPixmap.fromImage(self.image))
#         self.setScaledContents(True)
#
#     def paintEvent(self, event):
#         painter = QPainter(self)
#         painter.drawImage(event.rect(), self.image)
#         painter.end()
#
#     def drawSeedMark(self, x, y):
#         if sys.version_info.major == 2:
#             xx = (self.mask_points[0] + x) # .astype(np.int)
#             yy = (self.mask_points[1] + y) # .astype(np.int)
#         else:
#             xx = (self.mask_points[0] + x + 0.5) # .astype(np.int)
#             yy = (self.mask_points[1] + y + 0.5) # .astype(np.int)
#         idx = np.arange(len(xx))
#         idx[np.where(xx < 0)] = -1
#         idx[np.where(xx >= self.slice_size[0])] = -1
#         idx[np.where(yy < 0)] = -1
#         idx[np.where(yy >= self.slice_size[1])] = -1
#         ii = idx[np.where(idx >= 0)]
#         xx_ii = xx[ii]#.round().astype(np.int)
#         yy_ii = yy[ii]#.round().astype(np.int)
#
#         linear_index = (yy_ii * self.slice_size[0] + xx_ii).round().astype(np.int)
#
#         self.seeds[linear_index] = self.seed_mark
#
#     def drawLine(self, p0, p1):
#         """
#         Draw line to slice image and seed matrix.
#
#         Parameters
#         ----------
#         p0 : tuple of int
#             Line star point.
#         p1 : tuple of int
#             Line end point.
#         """
#
#         x0, y0 = p0
#         x1, y1 = p1
#         dx = np.abs(x1-x0)
#         dy = np.abs(y1-y0)
#         if x0 < x1:
#             sx = 1
#
#         else:
#             sx = -1
#
#         if y0 < y1:
#             sy = 1
#
#         else:
#             sy = -1
#
#         err = dx - dy
#
#         while True:
#             self.drawSeedMark(x0,y0)
#
#             if x0 == x1 and y0 == y1:
#                 break
#
#             e2 = 2*err
#             if e2 > -dy:
#                 err = err - dy
#                 x0 = x0 + sx
#
#             if e2 <  dx:
#                 err = err + dx
#                 y0 = y0 + sy
#
#     def drawSeeds(self, pos):
#         """
#
#         :param pos: list of two indexes with mouse position
#         :return:
#         """
#         if pos[0] < 0 or pos[0] >= self.slice_size[0] \
#                 or pos[1] < 0 or pos[1] >= self.slice_size[1]:
#             return
#
#         self.drawLine(self.last_position, pos)
#         self.updateSlice()
#
#         self.modified = True
#         self.last_position = pos
#
#         self.update()
#
#     def get_contours(self, img, sl):
#         idxs = sl.nonzero()[0]
#         keys = np.unique(sl[idxs])
#         for ii in keys:
#             if ii == 0:
#                 continue
#             aux = np.zeros_like(sl)
#             idxsi = np.where(sl == ii)[0]
#             aux[idxsi] = 1
#             cnt = self.gen_contours(aux)
#
#             self.composeRgba(img, cnt,
#                              CONTOURLINES_COLORTABLE[ii - 1,...])
#
#
#     def gen_contours(self, sl):
#         sls = sl.reshape(self.slice_size, order='F')
#         cnt = sls.copy()
#         chunk = np.zeros((cnt.shape[1] + 2,), dtype=np.int8)
#         for irow, row in enumerate(sls):
#             chunk[1:-1] = row
#             chdiff = np.diff(chunk)
#             idx1 = np.where(chdiff > 0)[0]
#             if idx1.shape[0] > 0:
#                 idx2 = np.where(chdiff < 0)[0]
#                 if idx2.shape[0] > 0:
#                     cnt[irow,idx1] = 2
#                     cnt[irow,idx2 - 1] = 2
#
#         chunk = np.zeros((cnt.shape[0] + 2,), dtype=np.int8)
#         for icol, col in enumerate(sls.T):
#             chunk[1:-1] = col
#             chdiff = np.diff(chunk)
#             idx1 = np.where(chdiff > 0)[0]
#             if idx1.shape[0] > 0:
#                 idx2 = np.where(chdiff < 0)[0]
#                 if idx2.shape[0] > 0:
#                     cnt[idx1,icol] = 2
#                     cnt[idx2 - 1,icol] = 2
#
#         return cnt.ravel(order='F')
#
#     def composeRgba(self, bg, fg, cmap):
#         # TODO here is bug. Should be used nearest neighboor instead of linear interpolation
#         idxs = fg.nonzero()[0]
#
#         if idxs.shape[0] > 0:
#             fg_rgb = cmap[fg[idxs] - 1]
#
#             af = fg_rgb[...,3].astype(np.uint32)
#             rgbf = fg_rgb[...,:3].astype(np.uint32)
#             rgbb = bg[idxs,:3].astype(np.uint32)
#
#             rgbx = ((rgbf.T * af).T + (rgbb.T * (255 - af)).T) / 255
#             bg[idxs,:3] = rgbx.astype(np.uint8)
#
#     def overRgba(self, bg, fg, cmap):
#         idxs = fg.nonzero()[0]
#         bg[idxs] = cmap[fg[idxs] - 1]
#
#     def updateSlice(self):
#
#         if self.ctslice_rgba is None:
#             return
#
#         img = self.ctslice_rgba.copy()
#
#         if self.seeds is not None:
#             if self.mode_draw:
#                 if self.contour_mode == 'fill':
#                     self.composeRgba(img, self.seeds,
#                                      self.seeds_colortable)
#                 elif self.contour_mode == 'contours':
#                     self.get_contours(img, self.seeds)
#
#             else:
#                 self.overRgba(img, self.seeds,
#                               self.seeds_colortable)
#
#         if self.contours is not None:
#             if self.contour_mode == 'fill':
#                 self.composeRgba(img, self.contours,
#                                  CONTOURS_COLORTABLE)
#
#             elif self.contour_mode == 'contours':
#                 self.get_contours(img, self.contours)
#
#         image = QImage(img.flatten(),
#                      self.slice_size[0], self.slice_size[1],
#                      QImage.Format_ARGB32).scaled(self.imagesize)
#         painter = QPainter(self.image)
#         painter.drawImage(0, 0, image)
#         painter.end()
#
#         self.update()
#
#     def getSliceRGBA(self, ctslice):
#         if self.cw['w'] > 0:
#             mul = 255.0 / float(self.cw['w'])
#
#         else:
#             mul = 0
#
#         lb = self.cw['c'] - self.cw['w'] / 2
#         aux = (ctslice.ravel(order='F') - lb) * mul
#         idxs = np.where(aux < 0)[0]
#         aux[idxs] = 0
#         idxs = np.where(aux > 255)[0]
#         aux[idxs] = 255
#
#         return aux.astype(np.uint8)
#
#     def updateSliceCW(self, ctslice=None):
#         if ctslice is not None:
#             self.ctslice_rgba = GRAY_COLORTABLE[self.getSliceRGBA(ctslice)]
#
#         self.updateSlice()
#
#     def setSlice(self, ctslice=None, seeds=None, contours=None):
#         if ctslice is not None:
#             self.ctslice_rgba = GRAY_COLORTABLE[self.getSliceRGBA(ctslice)]
#
#         if seeds is not None:
#             self.seeds = seeds.ravel(order='F')
#
#         else:
#             self.seeds = None
#
#         if contours is not None:
#             self.contours = contours.ravel(order='F')
#
#         else:
#             self.contours = None
#
#         self.updateSlice()
#
#     def getSliceSeeds(self):
#         if self.modified:
#             self.modified = False
#             return self.seeds.reshape(self.slice_size, order='F')
#
#         else:
#             return None
#
#     def gridPosition(self, pos):
#         return (int(pos.x() / self.grid[0]),
#                 int(pos.y() / self.grid[1]))
#
#     # mouse events
#     def mousePressEvent(self, event):
#         if event.button() in self.box_buttons:
#             self.drawing = True
#             modifiers = PyQt4.QtGui.QApplication.keyboardModifiers()
#             shift_offset = 0
#             if modifiers == PyQt4.QtCore.Qt.ShiftModifier:
#                 shift_offset = BOX_BUTTONS_SEED_SHIFT_OFFSET
#             elif modifiers == PyQt4.QtCore.Qt.ControlModifier:
#                 # this make seed_mark = 0 when left button is pressed
#                 shift_offset = -1
#
#             #     print('Control+Click')
#             # elif modifiers == (QtCore.Qt.ControlModifier |
#             #                    QtCore.Qt.ShiftModifier):
#             # print('Control+Shift+Click')
#             self.seed_mark = self.box_buttons[event.button()] + shift_offset
#             self.last_position = self.gridPosition(event.pos())
#
#         elif event.button() == Qt.MiddleButton:
#             self.drawing = False
#             self.erase_region_button = True
#
#     def mouseMoveEvent(self, event):
#         if self.drawing:
#             self.drawSeeds(self.gridPosition(event.pos()))
#
#     def mouseReleaseEvent(self, event):
#         if (event.button() in self.box_buttons) and self.drawing:
#             self.drawSeeds(self.gridPosition(event.pos()))
#             self.drawing = False
#
#         if event.button() == Qt.MiddleButton\
#           and self.erase_region_button == True:
#             self.eraseRegion(self.gridPosition(event.pos()),
#                              self.erase_mode)
#
#             self.erase_region_button == False
#
#     def resizeSlice(self, new_slice_size=None, new_grid=None):
#
#         if new_slice_size is not None:
#             self.slice_size = new_slice_size
#
#         if new_grid is not None:
#             self.grid = new_grid
#
#         self.imagesize = QSize(int(self.slice_size[0] * self.grid[0]),
#                                int(self.slice_size[1] * self.grid[1]))
#         self.image = QImage(self.imagesize, QImage.Format_RGB32)
#         self.setPixmap(QPixmap.fromImage(self.image))
#
#     def resizeEvent(self, event):
#         new_height = self.height()
#         new_grid = new_height / float(self.slice_size[1])
#         mul = new_grid / self.grid[1]
#
#         self.grid = np.array(self.grid) * mul
#         self.resizeSlice()
#         self.updateSlice()
#
#     def leaveEvent(self, event):
#         self.drawing = False
#
#     def enterEvent(self, event):
#         self.drawing = False
#         self.emit(SIGNAL('focus_slider'))
#
#     def setMaskPoints(self, mask):
#         self.mask_points = mask
#
#     def getCW(self):
#         return self.cw
#
#     def setCW(self, val, key):
#         self.cw[key] = val
#
#     def eraseRegion(self, pos, mode):
#         if self.erase_fun is not None:
#             self.erase_fun(pos, mode)
#             self.updateSlice()
#
#     def setEraseFun(self, fun):
#         self.erase_fun = fun
#
#     def setScrollFun(self, fun):
#         self.scroll_fun = fun
#
#     def wheelEvent(self, event):
#         d = event.delta()
#         nd = d / abs(d)
#         if self.scroll_fun is not None:
#             self.scroll_fun(-nd)
#
#
# # TODO do widget
# # class QTSeedEditorWidget(QWidget):
#
# class QTSeedEditor(QDialog):
#     """
#     DICOM viewer.
#     """
#
#
#     @staticmethod
#     def get_line(mode='h'):
#         line = QFrame()
#         if mode == 'h':
#             line.setFrameStyle(QFrame.HLine)
#         elif mode == 'v':
#             line.setFrameStyle(QFrame.VLine)
#
#         line.setSizePolicy(QSizePolicy.Minimum, QSizePolicy.Expanding)
#
#         return line
#
#     def initUI(self, shape, vscale, height=600,
#                mode='seed',
#                button_text=None,
#                button_callback=None,
#                appmenu_text=None,
#                ):
#         """
#         Initialize UI.
#
#         Parameters
#         ----------
#         shape : (int, int, int)
#             Shape of data matrix.
#         vscale : (float, float, float)
#             Voxel scaling.
#         height : int
#             Maximal slice height in pixels.
#         mode : str
#             Editor mode.
#         """
#
#         # picture
#         grid = height / float(shape[1] * vscale[1])
#         mgrid = (grid * vscale[0], grid * vscale[1])
#         self.slice_box = SliceBox(shape[:-1], mgrid,
#                                   mode)
#         self.slice_box.setScrollFun(self.scrollSlices)
#         self.connect(self.slice_box, SIGNAL('focus_slider'), self.focusSliceSlider)
#
#         # sliders
#         self.allow_select_slice = True
#         self.n_slices = shape[2]
#         self.slider = QSlider(Qt.Vertical)
#         self.slider.label = QLabel()
#         self.slider.label.setText('Slice: %d / %d' % (self.actual_slice,
#                                                       self.n_slices))
#         self.slider.setRange(1, self.n_slices)
#         self.slider.valueChanged.connect(self.sliderSelectSlice)
#         self.slider.setValue(self.actual_slice)
#
#         self.slider_cw = {}
#         self.slider_cw['c'] = QSlider(Qt.Horizontal)
#         self.slider_cw['c'].valueChanged.connect(self.changeC)
#         self.slider_cw['c'].label = QLabel()
#
#         self.slider_cw['w'] = QSlider(Qt.Horizontal)
#         self.slider_cw['w'].valueChanged.connect(self.changeW)
#         self.slider_cw['w'].label = QLabel()
#
#         self.view_label = QLabel('View size: %d x %d' % self.img_aview.shape[:-1])
#         self.voxel_label = QLabel('Voxel size [mm]:\n  %.2f x %.2f x %.2f'\
#                                       % tuple(self.voxel_size[np.array(self.act_transposition)]))
#
#         combo_view_options = list(VIEW_TABLE)
#         combo_view = QComboBox(self)
#         combo_view.activated[str].connect(self.setView)
#         combo_view.addItems(combo_view_options)
#
#         # buttons
#         self.btn_quit = QPushButton("Return", self)
#         self.btn_quit.clicked.connect(self.quit)
#
#         combo_dmask = QComboBox(self)
#         combo_dmask.activated.connect(self.changeMask)
#         self.mask_points_tab, aux = self.init_draw_mask(DRAW_MASK, mgrid)
#         for icon, label in aux:
#             combo_dmask.addItem(icon, label)
#
#
#         self.slice_box.setMaskPoints(self.mask_points_tab[combo_dmask.currentIndex()])
#
#         self.status_bar = QStatusBar()
#
#         self.seeds_copy = None
#         vopts = []
#         vmenu = []
#         appmenu = []
#         if mode == 'seed' and self.mode_fun is not None:
#             btn_recalc = QPushButton("Recalculate", self)
#             btn_recalc.clicked.connect(self.recalculate)
#             self.btn_save= QPushButton("Save seeds", self)
#             self.btn_save.clicked.connect(self.saveload_seeds)
#             btn_s2b= QPushButton("Seg. to bckgr.", self)
#             btn_s2b.clicked.connect(self.seg_to_background_seeds)
#             btn_s2f= QPushButton("Seg. to forgr.", self)
#             btn_s2f.clicked.connect(self.seg_to_foreground_seeds)
#             appmenu.append(QLabel('<b>Segmentation mode</b><br><br><br>' +
#                                   'Select the region of interest<br>' +
#                                   'using the mouse buttons:<br><br>' +
#                                   '&nbsp;&nbsp;<i>left</i> - inner region<br>' +
#                                   '&nbsp;&nbsp;<i>right</i> - outer region<br><br>'))
#             appmenu.append(btn_recalc)
#             appmenu.append(self.btn_save)
#             appmenu.append(btn_s2f)
#             appmenu.append(btn_s2b)
#             appmenu.append(QLabel())
#             self.volume_label = QLabel('Volume:\n  unknown')
#             appmenu.append(self.volume_label)
#
#             # Set middle pencil as default (M. Jirik)
#             combo_dmask.setCurrentIndex(1)
#             self.slice_box.setMaskPoints(
#                 self.mask_points_tab[combo_dmask.currentIndex()])
#             #  -----mjirik---end------
#
#         if mode == 'seed' or mode == 'crop'\
#                 or mode == 'mask' or mode == 'draw':
#
#             combo_seed_label_options = ['all', '1', '2', '3', '4']
#             combo_seed_label= QComboBox(self)
#             combo_seed_label.activated[str].connect(self.changeFocusedLabel)
#             combo_seed_label.addItems(combo_seed_label_options)
#             self.changeFocusedLabel(combo_seed_label_options[combo_seed_label.currentIndex()])
#             # vopts.append(QLabel('Label to delete:'))
#             # vopts.append(combo_seed_label)
#             vmenu.append(QLabel('Label to delete:'))
#             vmenu.append(combo_seed_label)
#
#             btn_del = QPushButton("Del Slice Seeds", self)
#             btn_del.clicked.connect(self.deleteSliceSeeds)
#             vmenu.append(None)
#             vmenu.append(btn_del)
#             btn_del = QPushButton("Del All Seeds", self)
#             btn_del.clicked.connect(self.deleteSeedsInAllImage)
#             vmenu.append(None)
#             vmenu.append(btn_del)
#
#             combo_contour_options = ['fill', 'contours', 'off']
#             combo_contour = QComboBox(self)
#             combo_contour.activated[str].connect(self.changeContourMode)
#             combo_contour.addItems(combo_contour_options)
#             self.changeContourMode(combo_contour_options[combo_contour.currentIndex()])
#             vopts.append(QLabel('Selection mode:'))
#             vopts.append(combo_contour)
#
#         if mode == 'mask':
#             if button_text is None:
#                 button_text = "Mask region"
#             if button_callback is None:
#                 button_callback = self.maskRegion
#
#             btn_recalc_mask = QPushButton("Recalculate mask", self)
#             btn_recalc_mask.clicked.connect(self.updateMaskRegion_btn)
#             btn_all = QPushButton("Select all", self)
#             btn_all.clicked.connect(self.maskSelectAll)
#             btn_reset = QPushButton("Reset selection", self)
#             btn_reset.clicked.connect(self.resetSelection)
#             btn_reset_seads = QPushButton("Reset seads", self)
#             btn_reset_seads.clicked.connect(self.resetSeads)
#             btn_add = QPushButton("Add selection", self)
#             btn_add.clicked.connect(self.maskAddSelection)
#             btn_rem = QPushButton("Remove selection", self)
#             btn_rem.clicked.connect(self.maskRemoveSelection)
#             btn_mask = QPushButton(button_text, self)
#             btn_mask.clicked.connect(button_callback)
#             appmenu.append(QLabel('<b>Mask mode</b><br><br><br>' +
#                                   'Select the region to mask<br>' +
#                                   'using the left mouse button<br><br>'))
#             appmenu.append(self.get_line('h'))
#             appmenu.append(btn_recalc_mask)
#             appmenu.append(btn_all)
#             appmenu.append(btn_reset)
#             appmenu.append(btn_reset_seads)
#             appmenu.append(self.get_line('h'))
#             appmenu.append(btn_add)
#             appmenu.append(btn_rem)
#             appmenu.append(self.get_line('h'))
#             appmenu.append(btn_mask)
#             appmenu.append(self.get_line('h'))
#             self.mask_qhull = None
#
#         if mode == 'crop':
#             btn_crop = QPushButton("Crop", self)
#             btn_crop.clicked.connect(self.crop)
#             appmenu.append(QLabel('<b>Crop mode</b><br><br><br>' +
#                                   'Select the crop region<br>' +
#                                   'using the left mouse button<br><br>'))
#             appmenu.append(btn_crop)
#
#         if mode == 'draw':
#             appmenu.append(QLabel('<b>Manual segmentation<br> mode</b><br><br><br>' +
#                                   'Mark the region of interest<br>' +
#                                   'using the mouse buttons:<br><br>' +
#                                   '&nbsp;&nbsp;<i>left</i> - draw<br>' +
#                                   '&nbsp;&nbsp;<i>right</i> - erase<br>' +
#                                   '&nbsp;&nbsp;<i>middle</i> - vol. erase<br><br>'))
#
#             btn_reset = QPushButton("Reset", self)
#             btn_reset.clicked.connect(self.resetSliceDraw)
#             vmenu.append(None)
#             vmenu.append(btn_reset)
#
#             combo_erase_options = ['inside', 'outside']
#             combo_erase = QComboBox(self)
#             combo_erase.activated[str].connect(self.changeEraseMode)
#             combo_erase.addItems(combo_erase_options)
#             self.changeEraseMode(combo_erase_options[combo_erase.currentIndex()])
#             vopts.append(QLabel('Volume erase mode:'))
#             vopts.append(combo_erase)
#
#         if appmenu_text is not None:
#                 appmenu.append(QLabel(appmenu_text))
#         hbox = QHBoxLayout()
#         vbox = QVBoxLayout()
#         vbox_left = QVBoxLayout()
#         vbox_app = QVBoxLayout()
#
#         hbox.addWidget(self.slice_box)
#         hbox.addWidget(self.slider)
#         vbox_left.addWidget(self.slider.label)
#         vbox_left.addWidget(self.view_label)
#         vbox_left.addWidget(self.voxel_label)
#         vbox_left.addWidget(QLabel())
#         vbox_left.addWidget(QLabel('View plane:'))
#         vbox_left.addWidget(combo_view)
#         vbox_left.addWidget(self.get_line())
#         vbox_left.addWidget(self.slider_cw['c'].label)
#         vbox_left.addWidget(self.slider_cw['c'])
#         vbox_left.addWidget(self.slider_cw['w'].label)
#         vbox_left.addWidget(self.slider_cw['w'])
#         vbox_left.addWidget(self.get_line())
#         vbox_left.addWidget(QLabel('Drawing mask:'))
#         vbox_left.addWidget(combo_dmask)
#
#         for ii in vopts:
#             vbox_left.addWidget(ii)
#
#         for ii in vmenu:
#             if ii is None:
#                 vbox_left.addStretch(1)
#
#             else:
#                 vbox_left.addWidget(ii)
#
#         for ii in appmenu:
#             if ii is None:
#                 vbox_app.addStretch(1)
#
#             else:
#                 vbox_app.addWidget(ii)
#
#         vbox_app.addStretch(1)
#         vbox_app.addWidget(self.btn_quit)
#
#         hbox.addLayout(vbox_left)
#         hbox.addWidget(self.get_line('v'))
#         hbox.addLayout(vbox_app)
#         vbox.addLayout(hbox)
#         vbox.addWidget(self.status_bar)
#         self.my_layout = vbox
#         self.setLayout(vbox)
#
#         self.setWindowTitle('Segmentation Editor')
#         self.show()
#
#     def __init__(self, img, viewPositions=None,
#                  seeds=None, contours=None,
#                  mode='seed', modeFun=None,
#                  voxelSize=[1,1,1], volume_unit='mm3',
#                  button_text=None,
#                  button_callback=None,
#                  appmenu_text=None,
#                  ):
#         """
#         Initiate Editor
#
#         Parameters
#         ----------
#         :param img: array
#             DICOM data matrix.
#         :param actualSlice : int
#             Index of actual slice.
#         :param seeds : array
#             Seeds, user defined regions of interest.
#         :param contours : array
#             Computed segmentation.
#         :param mode : str
#             Editor modes:
#                'seed' - seed editor
#                'crop' - manual crop
#                'draw' - drawing
#                'mask' - mask region
#         :param modeFun : fun
#             Mode function invoked by user button.
#         :param voxelSize : tuple of float
#             voxel size [mm]
#         :param volume_unit : allow select output volume in mililiters or mm3
#             [mm, ml]
#         :param appmenu_text: text which is displayed in the right toolbar
#         :param button_callback: callback function used when button is clicked. Implemented in
#         "mask" mode. If none, default mask function is used.
#         :param button_text: text on the button. Implemented for "mask" mode. If None, default text
#         is used.
#         """
#
#         QDialog.__init__(self)
#
#         self.BACKGROUND_NOMODEL_SEED_LABEL = 4
#         self.FOREGROUND_NOMODEL_SEED_LABEL = 3
#
#         self.mode = mode
#         self.mode_fun = modeFun
#
#         self.actual_view = 'axial'
#         self.act_transposition = VIEW_TABLE[self.actual_view]
#         self.img = img
#         self.img_aview = self.img.transpose(self.act_transposition)
#
#         self.volume_unit = volume_unit
#
#         self.last_view_position = {}
#         for jj, ii in enumerate(VIEW_TABLE.keys()):
#             if viewPositions is None:
#                 viewpos = img.shape[VIEW_TABLE[ii][-1]] / 2
#
#             else:
#                 viewpos = viewPositions[jj]
#
#             self.last_view_position[ii] =\
#                 img.shape[VIEW_TABLE[ii][-1]] - viewpos - 1
#
#         self.actual_slice = self.last_view_position[self.actual_view]
#
#         # set contours
#         self.contours = contours
#         if self.contours is None:
#             self.contours_aview = None
#         else:
#             self.contours_aview = self.contours.transpose(self.act_transposition)
#
#         # masked data - has information about which data were removed
#         # 1 == enabled, 0 == deleted
#         # How to return:
#         #       editorDialog.exec_()
#         #       masked_data = editorDialog.masked
#         self.masked = np.ones(self.img.shape, np.int8)
#
#         self.voxel_size = np.squeeze(np.asarray(voxelSize))
#         self.voxel_scale = self.voxel_size / float(np.min(self.voxel_size))
#         self.voxel_volume = np.prod(voxelSize)
#
#         # set seeds
#         if seeds is None:
#             self.seeds = np.zeros(self.img.shape, np.int8)
#         else:
#             self.seeds = seeds
#
#         self.seeds_aview = self.seeds.transpose(self.act_transposition)
#         self.seeds_modified = False
#
#         self.initUI(self.img_aview.shape,
#                     self.voxel_scale[np.array(self.act_transposition)],
#                     600, mode,
#                     button_text=button_text,
#                     button_callback=button_callback,
#                     appmenu_text=appmenu_text,
#                     )
#
#         if mode == 'draw':
#             self.seeds_orig = self.seeds.copy()
#             self.slice_box.setEraseFun(self.eraseVolume)
#
#         # set view window values C/W
#         lb = np.min(img)
#         self.img_min_val = lb
#         ub = np.max(img)
#         dul = np.double(ub) - np.double(lb)
#         self.cw_range = {'c': [lb, ub], 'w': [1, dul]}
#         self.slider_cw['c'].setRange(lb, ub)
#         self.slider_cw['w'].setRange(1, dul)
#         self.changeC(lb + dul / 2)
#         self.changeW(dul)
#
#         self.offset = np.zeros((3,), dtype=np.int16)
#         # set what labels will be deleted by 'delete seeds' button
#         self.textFocusedLabel = "all"
#
#     def showStatus(self, msg):
#         self.status_bar.showMessage(QString(msg))
#         QApplication.processEvents()
#
#     def init_draw_mask(self, draw_mask, grid):
#         mask_points = []
#         mask_iconlabel = []
#         for mask, label in draw_mask:
#             w, h = mask.shape
#             xx, yy = mask.nonzero()
#             mask_points.append((xx - w/2, yy - h/2))
#
#             img = QImage(w, h, QImage.Format_ARGB32)
#             img.fill(qRgba(255, 255, 255, 0))
#             for ii in range(xx.shape[0]):
#                 img.setPixel(xx[ii], yy[ii], qRgba(0, 0, 0, 255))
#
#             img = img.scaled(QSize(w * grid[0], h * grid[1]))
#             icon = QIcon(QPixmap.fromImage(img))
#             mask_iconlabel.append((icon, label))
#
#         return mask_points, mask_iconlabel
#
#     def saveSliceSeeds(self):
#         aux = self.slice_box.getSliceSeeds()
#         if aux is not None:
#             self.seeds_aview[..., self.actual_slice] = aux
#             self.seeds_modified = True
#
#         else:
#             self.seeds_modified = False
#
#     def updateMaskRegion_btn(self):
#         self.saveSliceSeeds()
#         self.updateMaskRegion()
#
#     def updateMaskRegion(self):
#         crp = self.getCropBounds(return_nzs=True)
#         if crp is not None:
#             off, cri, nzs = crp
#             if nzs[0].shape[0] <=5:
#                 self.showStatus("Not enough points (need >= 5)!")
#
#             else:
#                 points = np.transpose(nzs)
#                 hull = Delaunay(points)
#                 X, Y, Z = np.mgrid[cri[0], cri[1], cri[2]]
#                 grid = np.vstack([X.ravel(), Y.ravel(), Z.ravel()]).T
#                 simplex = hull.find_simplex(grid)
#                 fill = grid[simplex >=0,:]
#                 fill = (fill[:,0], fill[:,1], fill[:,2])
#                 if self.contours is None or self.contours_old is None:
#                     self.contours = np.zeros(self.img.shape, np.int8)
#                     self.contours_old = self.contours.copy()
#                 else:
#                     self.contours[self.contours != 2] = 0
#                 self.contours[fill] = 1
#                 self.contours_aview = self.contours.transpose(self.act_transposition)
#
#                 self.selectSlice(self.actual_slice)
#
#     def maskRegion(self):
#         self.masked[self.contours == 0] = 0
#
#         self.img[self.contours != 2] = self.img_min_val
#         self.contours.fill(0)
#         self.contours_old = self.contours.copy()
#         self.seeds.fill(0)
#         self.selectSlice(self.actual_slice)
#
#     def maskAddSelection(self):
#         self.updateMaskRegion()
#         if self.contours is None:
#             return
#
#         self.contours[self.contours == 1] = 2
#         self.contours_old = self.contours.copy()
#         self.seeds.fill(0)
#         self.selectSlice(self.actual_slice)
#
#     def maskRemoveSelection(self):
#         self.updateMaskRegion()
#         if self.contours is None:
#             return
#
#         self.contours[self.contours == 1] = 0
#         self.contours_old = self.contours.copy()
#         self.seeds.fill(0)
#         self.selectSlice(self.actual_slice)
#
#     def maskSelectAll(self):
#         self.updateMaskRegion()
#
#         self.seeds[0][0][0] = 1
#         self.seeds[0][0][-1] = 1
#         self.seeds[0][-1][0] = 1
#         self.seeds[0][-1][-1] = 1
#         self.seeds[-1][0][0] = 1
#         self.seeds[-1][0][-1] = 1
#         self.seeds[-1][-1][0] = 1
#         self.seeds[-1][-1][-1] = 1
#
#         self.updateMaskRegion()
#         self.selectSlice(self.actual_slice)
#
#     def resetSelection(self):
#         self.updateMaskRegion()
#         if self.contours is None:
#             return
#
#         self.contours.fill(0)
#         self.contours_old = self.contours.copy()
#         self.seeds.fill(0)
#         self.selectSlice(self.actual_slice)
#
#     def resetSeads(self):
#         self.seeds.fill(0)
#         if self.contours is not None:
#             self.contours = self.contours_old.copy()
#             self.contours_aview = self.contours.transpose(self.act_transposition)
#         self.updateMaskRegion()
#         self.selectSlice(self.actual_slice)
#
#     def updateCropBounds(self):
#         crp = self.getCropBounds()
#         if crp is not None:
#             _, cri = crp
#             self.contours = np.zeros(self.img.shape, np.int8)
#             self.contours[cri].fill(1)
#             self.contours_aview = self.contours.transpose(self.act_transposition)
#
#     def focusSliceSlider(self):
#         self.slider.setFocus(True)
#
#     def sliderSelectSlice(self, value):
#         self.selectSlice(self.n_slices - value)
#
#     def scrollSlices(self, inc):
#         if abs(inc) > 0:
#             new = self.actual_slice + inc
#             self.selectSlice(new)
#
#     def selectSlice(self, value, force=False):
#         if not(self.allow_select_slice):
#             return
#
#         if (value < 0) or (value >= self.n_slices):
#             return
#
#         if (value != self.actual_slice) or force:
#             self.saveSliceSeeds()
#             if self.seeds_modified:
#                 if self.mode == 'crop':
#                     self.updateCropBounds()
#                 elif self.mode == 'mask':
#                     self.updateMaskRegion()
#
#         if self.contours is None:
#             contours = None
#         else:
#             contours = self.contours_aview[..., int(value)]
#
#         slider_val = self.n_slices - value
#         self.slider.setValue(slider_val)
#         self.slider.label.setText('Slice: %d / %d' % (slider_val, self.n_slices))
#
#         self.slice_box.setSlice(self.img_aview[..., int(value)],
#                                 self.seeds_aview[..., int(value)],
#                                 contours)
#         self.actual_slice = int(value)
#
#     def getSeeds(self):
#         return self.seeds
#
#     def getImg(self):
#         return self.img
#
#     def getOffset(self):
#         return self.offset * self.voxel_size
#
#     def getSeedsVal(self, label):
#         return self.img[self.seeds==label]
#
#     def getContours(self):
#         return self.contours
#
#     def setContours(self, contours):
#         """
#         store segmentation
#         :param contours: segmentation
#         :return: Nothing
#         """
#         """
#         :param contours:
#         :return:
#         """
#         self.contours = contours
#         self.contours_aview = self.contours.transpose(self.act_transposition)
#
#         self.selectSlice(self.actual_slice)
#
#     def changeCW(self, value, key):
#         rg = self.cw_range[key]
#         if (value < rg[0]) or (value > rg[1]):
#             return
#
#         if (value != self.slice_box.getCW()[key]):
#             self.slider_cw[key].setValue(value)
#             self.slider_cw[key].label.setText('%s: %d' % (key.upper(), value))
#             self.slice_box.setCW(value, key)
#             self.slice_box.updateSliceCW(self.img_aview[...,self.actual_slice])
#
#
#     def changeC(self, value):
#         self.changeCW(value, 'c')
#
#     def changeW(self, value):
#         self.changeCW(value, 'w')
#
#     def setView(self, value):
#         self.last_view_position[self.actual_view] = self.actual_slice
#         # save seeds
#         self.saveSliceSeeds()
#         if self.seeds_modified:
#             if self.mode == 'crop':
#                 self.updateCropBounds()
#
#             elif self.mode == 'mask':
#                 self.updateMaskRegion()
#
#         key = str(value)
#         self.actual_view = key
#         self.actual_slice = self.last_view_position[key]
#
#         self.act_transposition = VIEW_TABLE[key]
#         self.img_aview = self.img.transpose(self.act_transposition)
#         self.seeds_aview = self.seeds.transpose(self.act_transposition)
#
#         if self.contours is not None:
#             self.contours_aview = self.contours.transpose(self.act_transposition)
#             contours = self.contours_aview[...,self.actual_slice]
#
#         else:
#             contours = None
#
#         vscale = self.voxel_scale[np.array(self.act_transposition)]
#         height = self.slice_box.height()
#         grid = height / float(self.img_aview.shape[1] * vscale[1])
#         # width = (self.img_aview.shape[0] * vscale[0])[0]
#         # if width > 800:
#         #     height = 400
#         #     grid = height / float(self.img_aview.shape[1] * vscale[1])
#
#         mgrid = (grid * vscale[0], grid * vscale[1])
#
#         self.slice_box.resizeSlice(new_slice_size=self.img_aview.shape[:-1],
#                                    new_grid=mgrid)
#
#         self.slice_box.setSlice(self.img_aview[...,self.actual_slice],
#                                 self.seeds_aview[...,self.actual_slice],
#                                 contours)
#
#         self.allow_select_slice = False
#         self.n_slices = self.img_aview.shape[2]
#         slider_val = self.n_slices - self.actual_slice
#         self.slider.setRange(1, self.n_slices)
#         self.slider.setValue(slider_val)
#         self.allow_select_slice = True
#
#         self.slider.label.setText('Slice: %d / %d' % (slider_val,
#                                                       self.n_slices))
#         self.view_label.setText('View size: %d x %d' % self.img_aview.shape[:-1])
#
#         self.adjustSize()
#         self.adjustSize()
#
#     def changeMask(self, val):
#         self.slice_box.setMaskPoints(self.mask_points_tab[val])
#
#     def changeContourMode(self, val):
#         self.slice_box.contour_mode = str(val)
#         self.slice_box.updateSlice()
#
#     def changeEraseMode(self, val):
#         self.slice_box.erase_mode = str(val)
#
#     def eraseVolume(self, pos, mode):
#         self.showStatus("Processing...")
#         xyz = np.array(pos + (self.actual_slice,))
#         p = np.zeros_like(xyz)
#         p[np.array(self.act_transposition)] = xyz
#         p = tuple(p)
#
#         if self.seeds[p] > 0:
#             if mode == 'inside':
#                 erase_reg(self.seeds, p, val=0)
#
#             elif mode == 'outside':
#                 erase_reg(self.seeds, p, val=-1)
#                 idxs = np.where(self.seeds < 0)
#                 self.seeds.fill(0)
#                 self.seeds[idxs] = 1
#
#         if self.contours is None:
#             contours = None
#
#         else:
#             contours = self.contours_aview[...,self.actual_slice]
#
#         self.slice_box.setSlice(self.img_aview[...,self.actual_slice],
#                                 self.seeds_aview[...,self.actual_slice],
#                                 contours)
#
#         self.showStatus("Done")
#
#     def cropUpdate(self, img):
#
#         for ii in VIEW_TABLE.keys():
#             self.last_view_position[ii] = 0
#         self.actual_slice = 0
#
#         self.img = img
#         self.img_aview = self.img.transpose(self.act_transposition)
#
#         self.contours = None
#         self.contours_aview = None
#
#         self.seeds = np.zeros(self.img.shape, np.int8)
#         self.seeds_aview = self.seeds.transpose(self.act_transposition)
#         self.seeds_modified = False
#
#         vscale = self.voxel_scale[np.array(self.act_transposition)]
#         height = self.slice_box.height()
#         grid = height / float(self.img_aview.shape[1] * vscale[1])
#         mgrid = (grid * vscale[0], grid * vscale[1])
#
#         self.slice_box.resizeSlice(new_slice_size=self.img_aview.shape[:-1],
#                                    new_grid=mgrid)
#
#         self.slice_box.setSlice(self.img_aview[...,self.actual_slice],
#                                 self.seeds_aview[...,self.actual_slice],
#                                 None)
#
#         self.allow_select_slice = False
#         self.n_slices = self.img_aview.shape[2]
#         self.slider.setValue(self.actual_slice + 1)
#         self.slider.setRange(1, self.n_slices)
#         self.allow_select_slice = True
#
#         self.slider.label.setText('Slice: %d / %d' % (self.actual_slice + 1,
#                                                       self.n_slices))
#         self.view_label.setText('View size: %d x %d' % self.img_aview.shape[:-1])
#
#     def getCropBounds(self, return_nzs=False, flat=False):
#
#         nzs = self.seeds.nonzero()
#         cri = []
#         flag = True
#         for ii in range(3):
#             if nzs[ii].shape[0] == 0:
#                 flag = False
#                 break
#
#             smin, smax = np.min(nzs[ii]), np.max(nzs[ii])
#             if not(flat):
#                 if smin == smax:
#                     flag = False
#                     break
#
#             cri.append((smin, smax))
#
#         if flag:
#             cri = np.array(cri)
#
#             out = []
#             offset = []
#             for jj, ii in enumerate(cri):
#                 out.append(slice(ii[0], ii[1] + 1))
#                 offset.append(ii[0])
#
#             if return_nzs:
#                 return np.array(offset), tuple(out), nzs
#
#             else:
#                 return np.array(offset), tuple(out)
#
#         else:
#             return None
#
#     def crop(self):
#         self.showStatus("Processing...")
#
#         crp = self.getCropBounds()
#
#         if crp is not None:
#             offset, cri = crp
#             crop = self.img[cri]
#             self.img = np.ascontiguousarray(crop)
#             self.offset += offset
#
#
#             self.showStatus('Done')
#
#         else:
#             self.showStatus('Region not selected!')
#
#         self.cropUpdate(self.img)
#
#     def seg_to_background_seeds(self, event):
#         self.saveSliceSeeds()
#         self.seeds[self.seeds < 3] = 0
#
#         from PyQt4.QtCore import pyqtRemoveInputHook
#         # pyqtRemoveInputHook()
#         # import ipdb; ipdb.set_trace()
#         self.seeds[(self.contours == 1) & (self.seeds < 3)] = self.BACKGROUND_NOMODEL_SEED_LABEL
#         self.contours[...] = 0
#
#     def seg_to_foreground_seeds(self, event):
#         self.saveSliceSeeds()
#         self.seeds[self.seeds < 3] = 0
#         # from PyQt4.QtCore import pyqtRemoveInputHook
#         # pyqtRemoveInputHook()
#         # import ipdb; ipdb.set_trace()
#         self.seeds[(self.contours == 1) & (self.seeds < 3)] = self.FOREGROUND_NOMODEL_SEED_LABEL
#         self.contours[...] = 0
#
#     def saveload_seeds(self, event):
#         if self.seeds_copy is None:
#             self.seeds_copy = self.seeds.copy()
#             self.seeds[...] = 0
#             # print "save"
#             # from PyQt4.QtCore import pyqtRemoveInputHook
#             # pyqtRemoveInputHook()
#             # import ipdb; ipdb.set_trace()
#             self.btn_save.setText("Load seeds")
#         else:
#             # from PyQt4.QtCore import pyqtRemoveInputHook
#             # pyqtRemoveInputHook()
#             # import ipdb; ipdb.set_trace()
#             self.seeds[self.seeds_copy > 0] = self.seeds_copy[self.seeds_copy > 0]
#             self.seeds_copy = None
#             self.btn_save.setText("Save seeds")
#
#
#     def recalculate(self, event):
#         self.saveSliceSeeds()
#         if np.abs(np.min(self.seeds) - np.max(self.seeds)) < 2:
#             self.showStatus('Inner and outer regions not defined!')
#             return
#
#         self.showStatus("Processing...")
#         self.mode_fun(self)
#         self.selectSlice(self.actual_slice)
#         self.updateVolume()
#         self.showStatus("Done")
#
#     def changeFocusedLabel(self, textlabel):
#         self.textFocusedLabel = textlabel
#         # logger
#         # print " lakjlfkj ", textlabel
#         logger.debug(self.textFocusedLabel)
#
#     def deleteSliceSeeds(self, event):
#         if self.textFocusedLabel == 'all':
#             self.seeds_aview[...,self.actual_slice] = 0
#         else:
#             # delete only seeds with specific label
#             self.seeds_aview[
#                 self.seeds_aview[...,self.actual_slice] == int(self.textFocusedLabel)
#                 ,self.actual_slice] = 0
#
#         self.slice_box.setSlice(seeds=self.seeds_aview[...,self.actual_slice])
#         self.slice_box.updateSlice()
#
#     def deleteSeedsInAllImage(self, event):
#         if self.textFocusedLabel == 'all':
#             self.seeds_aview[...] = 0
#         else:
#             # delete only seeds with specific label
#             self.seeds_aview[
#                 self.seeds_aview[...] == int(self.textFocusedLabel)
#                 ] = 0
#
#         self.slice_box.setSlice(seeds=self.seeds_aview[...,self.actual_slice])
#         self.slice_box.updateSlice()
#
#     def resetSliceDraw(self, event):
#         seeds_orig_aview = self.seeds_orig.transpose(self.act_transposition)
#         self.seeds_aview[...,self.actual_slice] = seeds_orig_aview[...,self.actual_slice]
#         self.slice_box.setSlice(seeds=self.seeds_aview[...,self.actual_slice])
#         self.slice_box.updateSlice()
#
#     def quit(self, event):
#         self.close()
#
#     def updateVolume(self):
#         text = 'Volume:\n  unknown'
#         if self.voxel_volume is not None:
#             if self.mode == 'draw':
#                 vd = self.seeds
#
#             else:
#                 vd = self.contours
#
#             if vd is not None:
#                 nzs = vd.nonzero()
#                 nn = nzs[0].shape[0]
#                 if self.volume_unit == 'ml':
#                     text = 'Volume [ml]:\n  %.2f' %\
#                         (nn * self.voxel_volume / 1000)
#                 else:
#                     text = 'Volume [mm3]:\n  %.2e' % (nn * self.voxel_volume)
#
#         self.volume_label.setText(text)
#
#     def getROI(self):
#         crp = self.getCropBounds()
#         if crp is not None:
#             _, cri = crp
#
#         else:
#             cri = []
#             for jj, ii in enumerate(self.img.shape):
#                 off = self.offset[jj]
#                 cri.append(slice(off, off + ii))
#
#         return cri
#
#
# # def old_int(x):
# #     """
# #     Python 2 style rounding.
# #     0.5 is rounded to 0. Not using banker's rounding.
# #
# #     :param x:
# #     :return: rounded value or array
# #     """
# #
# #     if sys.version_info.major == 2:
# #         return np.int(x)
# #     else:
# #         # return np.floor(np.abs(x) + 0.5) * np.sign(x)
# #         return np.int(x + .5)
#
# # def old_round(x):
# #     """
# #     Python 2 style rounding.
# #     0.5 is rounded to 0. Not using banker's rounding.
# #
# #     :param x:
# #     :return: rounded value or array
# #     """
# #
# #     if sys.version_info.major == 2:
# #         return round(x)
# #     else:
# #         # return np.floor(np.abs(x) + 0.5) * np.sign(x)
# #         return np.int(x + .5)
# #         # return np.floor(np.abs(x) + 0.5) * np.sign(x)
#
# def gen_test():
#     test = {}
#     # test['data'] = np.zeros((10,10,4), dtype=np.uint8)
#     test['data'] = (np.random.rand(4, 10, 9) * 100).astype(np.uint8)
#
#     test['voxelsize_mm'] = (2, 2, 2.5)
#     test['segdata'] = None
#
#     return test
#
# usage = '%prog [options]\n' + __doc__.rstrip()
# help = {
#     'in_file': 'input *.mat file with "data" field',
#     'mode': '"seed", "crop", "mask" or "draw" mode',
#     #'out_file': 'store the output matrix to the file',
#     #'debug': 'run in debug mode',
#     'gen_test': 'generate test data',
#     'test': 'run unit test',
# }
#
# def main():
#     parser = OptionParser(description='Segmentation editor')
#     parser.add_option('-f','--filename', action='store',
#                       dest='in_filename', default=None,
#                       help=help['in_file'])
#     # parser.add_option('-d', '--debug', action='store_true',
#     #                   dest='debug', help=help['debug'])
#     parser.add_option('-m', '--mode', action='store',
#                       dest='mode', default='seed', help=help['mode'])
#     parser.add_option('-t', '--tests', action='store_true',
#                       dest='unit_test', help=help['test'])
#     parser.add_option('-g', '--gener_data', action='store_true',
#                       dest='gen_test', help=help['gen_test'])
#
#     # parser.add_option('-o', '--outputfile', action='store',
#     #                   dest='out_filename', default='output.mat',
#     #                   help=help['out_file'])
#     (options, args) = parser.parse_args()
#
#     if options.gen_test:
#         dataraw = gen_test()
#
#     else:
#         if options.in_filename is None:
#             raise IOError('No input data!')
#
#         else:
#             dataraw = loadmat(options.in_filename,
#                               variable_names=['data', 'segdata',
#                                               'voxelsize_mm', 'seeds'])
#             if not ('segdata' in dataraw):
#                 dataraw['segdata'] = None
#
#     app = QApplication(sys.argv)
#     pyed = QTSeedEditor(dataraw['data'],
#                         # seeds=dataraw['segdata'],
#                         mode=options.mode,
#                         voxelSize=dataraw['voxelsize_mm'])
#     pyed.changeC(50)
#     pyed.changeW(50)
#     pyed.exec_()
#
#     # sys.exit(app.exec_())

if __name__ == "__main__":
    main()



#  /usr/bin/python
# -*- coding: utf-8 -*-

"""
DICOM reader

Example:

$ dcmreaddata -d sample_data -o head.mat
"""

from __future__ import print_function

import os
import numpy as np
from optparse import OptionParser
from scipy.io import savemat


import logging
logger = logging.getLogger(__name__)

from io3d.dcmreaddata import DicomReader, get_dcmdir_qt
# from io3d.misc import obj_from_file

# __version__ = [1, 3]

#
# def obj_from_file(filename='annotation.yaml', filetype='yaml'):
#     """
#     Read object from file
#     """
#     if filetype == 'yaml':
#         import yaml
#         f = open(filename, 'r')
#         obj = yaml.load(f)
#     elif filetype == 'pickle':
#         import pickle
#         f = open(filename, 'rb')
#         obj = pickle.load(f)
#     else:
#         logger.error('Unknown filetype')
#
#     f.close()
#
#     return obj
#
#
# def obj_to_file(obj, filename='annotation.yaml', filetype='yaml'):
#     """
#     Writes annotation in file
#     """
#     if filetype == 'yaml':
#         import yaml
#         f = open(filename, 'w')
#         yaml.dump(obj, f)
#     elif filetype == 'pickle':
#         import pickle
#         f = open(filename, 'wb')
#         pickle.dump(obj, f, -1)
#     else:
#         logger.error('Unknown filetype')
#
#     f.close
#
#
# def is_dicom_dir(datapath):
#     """
#     Check if in dir is one or more dicom file. We use two methods.
#     First is based on dcm extension detection.
#     """
#     # Second tries open files
#     # with dicom module.
#
#     retval = False
#     for f in os.listdir(datapath):
#         if f.endswith((".dcm", ".DCM")):
#             retval = True
#             return True
# # @todo not working and I dont know why
#         try:
#             dicom.read_file(f)
#
#             retval = True
#         except:
#             import traceback
#             traceback.print_exc
#             pass
#
#         if retval:
#             return True
#             print(f)
#     return False
#
#
# class DicomReader():
#     """
#     Example:
#
#     dcr = DicomReader(os.path.abspath(dcmdir))
#     data3d = dcr.get_3Ddata()
#     metadata = dcr.get_metaData()
#
#     """
#     dicomdir_filename = 'dicomdir.pkl'
#
#     def __init__(self, dirpath=None, initdir='.',
#                  qt_app=None, gui=True, series_number=None):
#         self.valid = False
#         self.dirpath = dirpath
#         self.dcmdir = self.get_dir()
#         self.series_number = series_number
#         self.overlay = {}
#         self.dcmlist = []
#
#         if len(self.dcmdir) > 0:
#             self.valid = True
#             counts, bins = self.status_dir()
#
#             if len(bins) > 1:
#                 if self.series_number is None:
#                     if (qt_app is not None) or gui:
#                         if qt_app is None:
#
#                             # @TODO  there is problem with type of qappliaction
#                             # mport PyQt4
#                             # rom PyQt4.QtGui import QApplication
#                             # t_app = QApplication(sys.argv)
#                             # t_app = PyQt4.QtGui.QWidget(sys.argv)
#                             print(qt_app)
#
#                         from PyQt4.QtGui import QInputDialog
#                         # bins = ', '.join([str(ii) for ii in bins])
#                         sbins = [str(ii) for ii in bins]
#                         snstring, ok = \
#                             QInputDialog.getItem(qt_app,
#                                                  'Serie Selection',
#                                                  'Select serie:',
#                                                  sbins,
#                                                  editable=False)
#                     else:
#                         print('series')
#                         series_info = self.dcmdirstats()
#                         print(self.print_series_info(series_info))
#                         snstring = input('Select Serie: ')
#
#                     sn = int(snstring)
#                 else:
#                     sn = self.series_number
#
#             else:
#                 sn = bins[0]
#
#             self.series_number = sn
#
#             self.dcmlist = self.get_sortedlist(SeriesNumber=sn)
#
#     def validData(self):
#         return self.valid
#
#     def get_overlay(self):
#         """
#         Function make 3D data from dicom file slices. There are usualy
#         more overlays in the data.
#         """
#         overlay = {}
#         dcmlist = self.dcmlist
#
#         for i in range(len(dcmlist)):
#             onefile = dcmlist[i]
#             logger.info(onefile)
#             data = dicom.read_file(onefile)
#
#             if len(overlay) == 0:
#                 # first there is created dictionary with
#                 # avalible overlay indexes
#                 for i_overlay in range(0, 50):
#                     try:
#                         # overlay index
#                         data2d = self.decode_overlay_slice(data, i_overlay)
#                         # mport pdb; pdb.set_trace()
#                         shp2 = data2d.shape
#                         overlay[i_overlay] = np.zeros([len(dcmlist), shp2[0],
#                                                       shp2[1]], dtype=np.int8)
#                         overlay[i_overlay][-i - 1, :, :] = data2d
#
#                     except:
#                         # exception is exceptetd. We are trying numbers 0-50
#                         # ogger.warning('Problem with overlay image number ' +
#                         #               str(i_overlay))
#                         pass
#
#             else:
#                 for i_overlay in overlay.keys():
#                     try:
#                         data2d = self.decode_overlay_slice(data, i_overlay)
#                         overlay[i_overlay][-i - 1, :, :] = data2d
#                     except:
#                         logger.warning('Problem with overlay number ' +
#                                        str(i_overlay))
#
#         return overlay
#
#     def decode_overlay_slice(self, data, i_overlay):
#             # overlay index
#             n_bits = 8
#
#             # On (60xx,3000) are stored ovelays.
#             # First is (6000,3000), second (6002,3000), third (6004,3000),
#             # and so on.
#             dicom_tag1 = 0x6000 + 2 * i_overlay
#
#             overlay_raw = data[dicom_tag1, 0x3000].value
#
#             # On (60xx,0010) and (60xx,0011) is stored overlay size
#             rows = data[dicom_tag1, 0x0010].value  # rows = 512
#             cols = data[dicom_tag1, 0x0011].value  # cols = 512
#
#             decoded_linear = np.zeros(len(overlay_raw) * n_bits)
#
#             # Decoding data. Each bit is stored as array element
# # TODO neni tady ta jednička blbě?
#             for i in range(1, len(overlay_raw)):
#                 for k in range(0, n_bits):
#                     byte_as_int = ord(overlay_raw[i])
#                     decoded_linear[i * n_bits + k] = (byte_as_int >> k) & 0b1
#
#             # verlay = np.array(pol)
#             overlay_slice = np.reshape(decoded_linear, [rows, cols])
#             return overlay_slice
#
#     def get_3Ddata(self, start=0, stop=None, step=1):
#         """
#         Function make 3D data from dicom file slices
#         """
#         data3d = []
#         dcmlist = self.dcmlist
#
#         if stop is None:
#             stop = len(dcmlist)
#
#         raw_max = None
#         raw_min = None
#         slope = None
#         inter = None
#
#         printRescaleWarning = False
#         for i in range(start, stop, step):
#             onefile = dcmlist[i]
#             logger.info(onefile)
#             data = dicom.read_file(onefile)
#             data2d = data.pixel_array
#             # mport pdb; pdb.set_trace()
#
#             if len(data3d) == 0:
#                 shp2 = data2d.shape
#                 data3d = np.zeros([len(dcmlist), shp2[0], shp2[1]],
#                                   dtype=np.int16)
#                 mx = np.max(data2d)
#                 mn = np.min(data2d)
#                 if (raw_max is None) or (raw_max < mx):
#                     raw_max = mx
#                 if (raw_min is None) or (raw_min < mx):
#                     raw_min = mn
#
#             try:
#                 slope = data.RescaleSlope
#                 inter = data.RescaleIntercept
#                 if slope == 1 and inter == 0:
#                     printRescaleWarning = True
#                     slope = 0.5
#                     inter = 0  # -16535
#                 new_data2d = (np.float(slope) * data2d)\
#                     + np.float(inter)
#
#             except:
#                 logger.warning(
#                     'problem with RescaleSlope and RescaleIntercept'
#                 )
#                 print('problem with RescaleSlope and RescaleIntercept')
#                 traceback.print_exc()
#                 print('----------')
#             # first readed slide is at the end
#
#             data3d[-i - 1, :, :] = new_data2d
#
#             logger.debug("Data size: " + str(data3d.nbytes)
#                          + ', shape: ' + str(shp2) + 'x' + str(len(dcmlist)))
#         if printRescaleWarning:
#             print("Automatic Rescale with slope 0.5")
#             logger.warning("Automatic Rescale with slope 0.5")
#
#         return data3d
#
#     def get_metaData(self, dcmlist=None, ifile=0):
#         """
#         Get metadata.
#         Voxel size is obtained from PixelSpacing and difference of
#         SliceLocation of two neighboorhoding slices (first have index ifile).
#         Files in are used.
#         """
#         if dcmlist is None:
#             dcmlist = self.dcmlist
#
#         if len(dcmlist) <= 0:
#             return {}
#
#         data = dicom.read_file(dcmlist[ifile])
#         try:
#             data2 = dicom.read_file(dcmlist[ifile + 1])
#             voxeldepth = float(
#                 np.abs(data.SliceLocation - data2.SliceLocation)
#             )
#         except:
#             logger.warning('Problem with voxel depth. Using SliceThickness,'
#                            + ' SeriesNumber: ' + str(data.SeriesNumber))
#
#             try:
#                 voxeldepth = float(data.SliceThickness)
#             except:
#                 logger.warning('Probem with SliceThicknes, setting zero. '
#                                + traceback.format_exc())
#                 voxeldepth = 0
#
#         pixelsize_mm = data.PixelSpacing
#         voxelsize_mm = [
#             voxeldepth,
#             float(pixelsize_mm[0]),
#             float(pixelsize_mm[1]),
#         ]
#         metadata = {'voxelsize_mm': voxelsize_mm,
#                     'Modality': data.Modality,
#                     'SeriesNumber': self.series_number
#                     }
#
#         try:
#             metadata['SeriesDescription'] = data.SeriesDescription
#
#         except:
#             logger.warning(
#                 'Problem with tag SeriesDescription, SeriesNumber: ' +
#                 str(data.SeriesNumber))
#         try:
#             metadata['ImageComments'] = data.ImageComments
#         except:
#             logger.warning(
#                 'Problem with tag ImageComments, SeriesNumber: ' +
#                 str(data.SeriesNumber))
#         try:
#             metadata['Modality'] = data.Modality
#         except:
#             logger.warning(
#                 'Problem with tag Modality, SeriesNumber: ' +
#                 str(data.SeriesNumber))
#
#         metadata['dcmfilelist'] = self.dcmlist
#
#         # mport pdb; pdb.set_trace()
#         return metadata
#
#     def dcmdirstats(self):
#         """ Dicom series staticstics, input is dcmdir, not dirpath
#         Information is generated from dicomdir.pkl and first files of series
#         """
#         import numpy as np
#         dcmdir = self.dcmdir
#         # get series number
# # vytvoření slovníku, kde je klíčem číslo série a hodnotou jsou všechny
# # informace z dicomdir
#         series_info = {line['SeriesNumber']: line for line in dcmdir}
#
# # počítání velikosti série
#         try:
#             dcmdirseries = [line['SeriesNumber'] for line in dcmdir]
#
#         except:
#             logger.debug('Dicom tag SeriesNumber not found')
#             series_info = {0: {'Count': 0}}
#             return series_info
#             # eturn [0],[0]
#
#         bins = np.unique(dcmdirseries)
#         binslist = bins.tolist()
# #  kvůli správným intervalům mezi biny je nutno jeden přidat na konce
#         mxb = np.max(bins) + 1
#         binslist.append(mxb)
#         # inslist.insert(0,-1)
#         counts, binsvyhodit = np.histogram(dcmdirseries, bins=binslist)
#
#         # db.set_trace();
#
#         # sestavení informace o velikosti série a slovníku
#
#         for i in range(0, len(bins)):
#             series_info[bins[i]]['Count'] = counts[i]
#
#             # adding information from files
#             lst = self.get_sortedlist(SeriesNumber=bins[i])
#             metadata = self.get_metaData(dcmlist=lst)
# # adding dictionary metadata to series_info dictionary
#             series_info[bins[i]] = dict(
#                 series_info[bins[i]].items() +
#                 metadata.items()
#             )
#
#         return series_info
#
#     def print_series_info(self, series_info):
#         """
#         Print series_info from dcmdirstats
#         """
#         strinfo = ''
#         if len(series_info) > 1:
#             for serie_number in series_info.keys():
#                 strl = str(serie_number) + " ("\
#                     + str(series_info[serie_number]['Count'])
#                 try:
#                     strl = strl + ", "\
#                         + str(series_info[serie_number]['Modality'])
#                     strl = strl + ", "\
#                         + str(series_info[serie_number]['SeriesDescription'])
#                     strl = strl + ", "\
#                         + str(series_info[serie_number]['ImageComments'])
#                 except:
#                     logger.debug(
#                         'Tag Modlity or ImageComment not found in dcminfo'
#                     )
#                     pass
#
#                 strl = strl + ')'
#                 strinfo = strinfo + strl + '\n'
#                 # rint strl
#
#         return strinfo
#
#     def files_in_dir(self, dirpath, wildcard="*", startpath=None):
#         """
#         Function generates list of files from specific dir
#
#         files_in_dir(dirpath, wildcard="*.*", startpath=None)
#
#         dirpath: required directory
#         wilcard: mask for files
#         startpath: start for relative path
#
#         Example
#         files_in_dir('medical/jatra-kiv','*.dcm', '~/data/')
#         """
#
#         import glob
#
#         filelist = []
#
#         if startpath is not None:
#             completedirpath = os.path.join(startpath, dirpath)
#         else:
#             completedirpath = dirpath
#
#         if os.path.exists(completedirpath):
#             logger.info('completedirpath = ' + completedirpath)
#
#         else:
#             logger.error('Wrong path: ' + completedirpath)
#             raise Exception('Wrong path : ' + completedirpath)
#
#         for infile in glob.glob(os.path.join(completedirpath, wildcard)):
#             filelist.append(infile)
#
#         if len(filelist) == 0:
#             logger.error('No required files in path: ' + completedirpath)
#             raise Exception('No required file in path: ' + completedirpath)
#
#         return filelist
#
#     def get_dir(self, writedicomdirfile=True):
#         """
#         Check if exists dicomdir file and load it or cerate it
#
#         dcmdir = get_dir(dirpath)
#
#         dcmdir: list with filenames, SeriesNumber and SliceLocation
#         """
#         createdcmdir = True
#
#         dicomdirfile = os.path.join(self.dirpath, self.dicomdir_filename)
#         ftype = 'pickle'
#         # if exist dicomdir file and is in correct version, use it
#         if os.path.exists(dicomdirfile):
#             dcmdirplus = obj_from_file(dicomdirfile, ftype)
#             try:
#                 if dcmdirplus['version'] == __version__:
#                     createdcmdir = False
#                 dcmdir = dcmdirplus['filesinfo']
#             except:
#                 logger.debug('Found dicomdir.pkl with wrong version')
#                 pass
#
#         if createdcmdir:
#             dcmdirplus = self.create_dir()
#             dcmdir = dcmdirplus['filesinfo']
#             if (writedicomdirfile) and len(dcmdir) > 0:
#                 obj_to_file(dcmdirplus, dicomdirfile, ftype)
#                 # bj_to_file(dcmdir, dcmdiryamlpath )
#
#         dcmdir = dcmdirplus['filesinfo']
#         return dcmdir
#
#     def create_dir(self):
#         """
#         Function crates list of all files in dicom dir with all IDs
#         """
#
#         filelist = self.files_in_dir(self.dirpath)
#         files = []
#
#         for filepath in filelist:
#             head, teil = os.path.split(filepath)
#             try:
#                 dcmdata = dicom.read_file(filepath)
#                 metadataline = {'filename': teil,
#                                 'SeriesNumber': dcmdata.SeriesNumber,
#                                 'SliceLocation': float(dcmdata.SliceLocation),
#                                 }
#
#                 # ry:
#                 #    metadataline ['ImageComment'] = dcmdata.ImageComments
#                 #    metadataline ['Modality'] = dcmdata.Modality
#                 # xcept:
#                 #    print 'Problem with ImageComments and Modality tags'
#
#                 files.append(metadataline)
#
#             # xcept Exception as e:
#             except:
#                 if head != self.dicomdir_filename:
#                     print('Dicom read problem with file ' + filepath)
#
#         files.sort(key=lambda x: x['SliceLocation'])
#
#         dcmdirplus = {'version': __version__, 'filesinfo': files}
#         return dcmdirplus
#
#     def status_dir(self):
#         """input is dcmdir, not dirpath """
#
#         try:
#             dcmdirseries = [line['SeriesNumber'] for line in self.dcmdir]
#         except:
#             return [0], [0]
#
#         bins = np.unique(dcmdirseries)
#         binslist = bins.tolist()
#         #  kvůli správným intervalům mezi biny je nutno jeden přidat na konce
#         mxb = np.max(bins) + 1
#         binslist.append(mxb)
#         counts, binsvyhodit = np.histogram(dcmdirseries, bins=binslist)
#
#         return counts, bins
#
#     def get_sortedlist(self, startpath="", SeriesNumber=None):
#         """
#         Function returns sorted list of dicom files. File paths are organized
#         by SeriesUID, StudyUID and FrameUID
#
#         Example:
#         get_sortedlist()
#         get_sortedlist('~/data/')
#         """
#         dcmdir = self.dcmdir[:]
#         dcmdir.sort(key=lambda x: x['SliceLocation'])
#
#         # select sublist with SeriesNumber
#         if SeriesNumber is not None:
#             dcmdir = [
#                 line for line in dcmdir if line['SeriesNumber'] == SeriesNumber
#             ]
#
#         logger.debug('SeriesNumber: ' + str(SeriesNumber))
#
#         filelist = []
#         for onefile in dcmdir:
#             filelist.append(os.path.join(startpath,
#                                          self.dirpath, onefile['filename']))
#             head, tail = os.path.split(onefile['filename'])
#
#         return filelist
#
#
# def get_dcmdir_qt(app=False, directory=''):
#     from PyQt4.QtGui import QFileDialog, QApplication
#     if app:
#         dcmdir = QFileDialog.getExistingDirectory(
#             caption='Select DICOM Folder',
#             options=QFileDialog.ShowDirsOnly,
#             directory=directory
#         )
#     else:
#         app = QApplication(sys.argv)
#         dcmdir = QFileDialog.getExistingDirectory(
#             caption='Select DICOM Folder',
#             options=QFileDialog.ShowDirsOnly,
#             directory=directory
#         )
#         # pp.exec_()
#         app.exit(0)
#     if len(dcmdir) > 0:
#
#         dcmdir = "%s" % (dcmdir)
#         dcmdir = dcmdir.encode("utf8")
#     else:
#         dcmdir = None
#     return dcmdir


usage = '%prog [options]\n' + __doc__.rstrip()
help = {
    'dcm_dir': 'DICOM data direcotory',
    'out_file': 'store the output matrix to the file',
    "degrad": "degradation of input data. For no degradation use 1",
    'debug': 'Print debug info',
    'zoom': 'Resize input data wit defined zoom. Use zoom 0.5 to obtain half\
voxels. Various zoom can be used for each axis: -z [1,0.5,2.5]'
}

if __name__ == "__main__":
    parser = OptionParser(description='Read DIOCOM data.')
    parser.add_option('-i', '--dcmdir', action='store',
                      dest='dcmdir', default=None,
                      help=help['dcm_dir'])
    parser.add_option('-o', '--outputfile', action='store',
                      dest='out_filename', default='output.mat',
                      help=help['out_file'])
    parser.add_option('--degrad', action='store',
                      dest='degrad', default=1,
                      help=help['degrad'])
    parser.add_option('-z', '--zoom', action='store',
                      dest='zoom', default='1',
                      help=help['zoom'])
    parser.add_option('-d', '--debug', action='store_true',
                      dest='debug',
                      help=help['debug'])
    (options, args) = parser.parse_args()

    logger.setLevel(logging.WARNING)
    ch = logging.StreamHandler()
    logger.addHandler(ch)

    if options.debug:
        logger.setLevel(logging.DEBUG)

    if options.dcmdir is None:
        dcmdir = get_dcmdir_qt()
        if dcmdir is None:
            raise IOError('No DICOM directory!')
    else:
        dcmdir = options.dcmdir

    dcr = DicomReader(os.path.abspath(dcmdir), gui=True)
    data3d = dcr.get_3Ddata()
    metadata = dcr.get_metaData()

    zoom = eval(options.zoom)
    if options.zoom != 1:
        import scipy
        import scipy.ndimage
        # oom = float(options.zoom)
        # mport pdb; pdb.set_trace()
        data3d = scipy.ndimage.zoom(data3d, zoom=zoom)
        metadata['voxelsize_mm'] = list(np.array(metadata['voxelsize_mm']) /
                                        zoom)

    degrad = int(options.degrad)

    data3d_out = data3d[::degrad, ::degrad, ::degrad]
    vs_out = list(np.array(metadata['voxelsize_mm']) * degrad)

    logger.debug('voxelsize_mm ' + vs_out.__str__())
    savemat(options.out_filename,
            {'data': data3d_out, 'voxelsize_mm': vs_out}
            )
    print("Data size: %d, shape: %s" % (data3d_out.nbytes, data3d_out.shape))



#! /usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import print_function

# import funkcí z jiného adresáře
import sys
import os.path
import unittest
import scipy
import numpy as np

import logging
logger = logging.getLogger(__name__)

path_to_script = os.path.dirname(os.path.abspath(__file__))
sys.path.append(os.path.join(path_to_script, "../src/"))

from nose.plugins.attrib import attr
from imcut import models


class ModelsTest(unittest.TestCase):
    # @classmethod
    # def setUpClass(cls):
    #     if sys.version_info.major < 3:
    #         cls.assertCountEqual = cls.assertItemsEqual

    def test_sigmoid(self):
        # from imcut import models
        import matplotlib.pyplot as plt
        x = np.linspace(-1000,1000, 100)
        min = 20
        max = 100
        multiplicator = max - min
        val = multiplicator * models.sigmoid(min + x * 1.0/multiplicator)
        # plt.plot(x, val)
        # plt.show()
        self.assertTrue(np.all(val >= 0))

    def test_softplus(self):
        import matplotlib.pyplot as plt
        x = np.linspace(-1000,1000, 1000)
        maximum_error = 10
        val = models.softplus(x, maximum_error)
        # plt.plot(x, val)
        # plt.show()
        self.assertTrue(np.all(val >= 0))
        self.assertTrue(np.all(val[x < 0] < maximum_error))
        # print(val)

if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
# import os.path as op

logger = logging.getLogger(__name__)
import numpy as nm
import numpy as np
import copy
# from io import open

# TODO zpětná indexace původních pixelů (v add_nodes())
# TODO nastavení velikosti bloku (v sr_tab)
# TODO funguje to ve 3D?
# TODO možnost vypínání zápisu do VTK (mjirik)
# TODO možnost kontroly jmena souborů do VTK(mjirik)
#
# TODO Jeden nový uzel reprezentuje více voxelů v původním obrázku
# TODO indexy vytvářet ve split_voxel()?
# TODO co je igrp? split_voxel()
# TODO Uzly se jen přidávají, při odběru se nastaví flag na False? Dořešuje se to ve finish()
# TODO co je ndid ve split_voxel()
# TODO Přidat relabeling ve finish
# TODO zápis do VTK souboru? line 381, #  f = open(fname, 'w') # UnicodeDecodeError: 'ascii' codec can't decode byte 0xcd in position 0: ordinal not in range(128)
#
# data = nm.array([[0,0,0],
#                  [0,1,1],
#                  [0,1,1],
#                  [1,0,1]])

class Graph(object):

    # spliting reconnection table
    # sr_tab = {
    #     2: nm.array([(0,2), (0,1), (1,3), (2,3)]),
    #     3: nm.array([(0,3,6), (0,1,2), (2,5,8), (6,7,8)]),
    #     4: nm.array([(0,4,8,12), (0,1,2,3), (3,7,11,15), (12,13,14,15)]),
    # }

    def add_nodes(self, coors, node_low_or_high=None):
        """
        Add new nodes at the end of the list.
        """
        last = self.lastnode
        if type(coors) is nm.ndarray:
            if len(coors.shape) == 1:
                coors = coors.reshape((1, coors.size))

            nadd = coors.shape[0]
            idx = slice(last, last + nadd)
        else:
            nadd = 1
            idx = self.lastnode
        right_dimension = coors.shape[1]
        self.nodes[idx, :right_dimension] = coors
        self.node_flag[idx] = True
        self.lastnode += nadd
        self.nnodes += nadd

    def add_edges(self, conn, edge_direction, edge_group=None, edge_low_or_high=None):
        """
        Add new edges at the end of the list.
        :param edge_direction: direction flag
        :param edge_group: describes group of edges from same low super node and same direction
        :param edge_low_or_high: zero for low to low resolution, one for high to high or high to low resolution.
        It is used to set weight from weight table.
        """
        last = self.lastedge
        if type(conn) is nm.ndarray:
            nadd = conn.shape[0]
            idx = slice(last, last + nadd)
            if edge_group is None:
                edge_group = nm.arange(nadd) + last
        else:
            nadd = 1
            idx = nm.array([last])
            conn = nm.array(conn).reshape((1, 2))
            if edge_group is None:
                edge_group = idx

        self.edges[idx, :] = conn
        self.edge_flag[idx] = True
        self.edge_dir[idx] = edge_direction
        self.edge_group[idx] = edge_group
        # TODO change this just to array of low_or_high_resolution
        if edge_low_or_high is not None and self._edge_weight_table is not None:
            self.edges_weights[idx] = self._edge_weight_table[edge_low_or_high, edge_direction]
        self.lastedge += nadd
        self.nedges += nadd


    def finish(self):
        ndidxs = nm.where(self.node_flag)[0]
        aux = - nm.ones((self.nodes.shape[0],), dtype=int)
        aux[ndidxs] = nm.arange(ndidxs.shape[0])
        edges = aux[self.edges[self.edge_flag]]
        nodes = self.nodes[ndidxs]

        # if self.compute_low_node_inverse:
        #     self.low_node_inverse = np.nonzero(self.nodes[:self.data.size])

        if self.compute_msindex:
            # self.msindex = self.msi.msindex
            # relabel
            self.msinds = aux[self.msi.msinds]
            # import sed3
            # ed = sed3.sed3(self.msindex)
            # ed.show()
            # np.unique(self.msindex.ravel())

        if self._edge_weight_table is not None:
            self.edges_weights = aux[self.edges_weights[self.edge_flag]]
            # del self.edges_weights
            # self.edges_weights = edges_weights


        del self.nodes
        del self.node_flag
        del self.edges
        del self.edge_flag
        del self.edge_dir
        del self.edge_group
        del self.msi

        self.nodes = nodes
        self.edges = edges
        self.node_flag = nm.ones((nodes.shape[0],), dtype=nm.bool)
        self.edge_flag = nm.ones((edges.shape[0],), dtype=nm.bool)

    def write_vtk(self, fname):
        write_grid_to_vtk(fname, self.nodes, self.edges, self.node_flag, self.edge_flag)

    def edges_by_group(self, idxs):
        """

        :param idxs: low resolution edge id
        :return: multiscale edges. If this part remain in low resolution the output is just one number
        """
        ed = self.edge_group[idxs]
        ugrps = nm.unique(ed)
        out = []
        for igrp in ugrps:
            out.append(idxs[nm.where(ed == igrp)[0]])

        return out

    def _edge_group_substitution(self, ndid, nsplit, idxs, sr_tab, ndoffset, ed_remove, into_or_from):
        """
        Reconnect edges.
        :param ndid: id of low resolution edges
        :param nsplit: number of split
        :param idxs: indexes of low resolution
        :param sr_tab:
        :param ndoffset:
        :param ed_remove:
        :param into_or_from: if zero, connection of input edges is done. If one, connection of output edges
        is performed.
        :return:
        """
        eidxs = idxs[nm.where(self.edges[idxs, 1 - into_or_from] == ndid)[0]]
        for igrp in self.edges_by_group(eidxs):
            if igrp.shape[0] > 1:
                # high resolution block to high resolution block
                # all directions are the same
                directions = self.edge_dir[igrp[0]]
                edge_indexes = sr_tab[directions, :].T.flatten() \
                                     + ndoffset
                # debug code
                # if len(igrp) != len(edge_indexes):
                #     print("Problem ")
                self.edges[igrp, 1] = edge_indexes
                if self._edge_weight_table is not None:
                    self.edges_weights[igrp] = self._edge_weight_table[1, directions]
            else:
                # low res block to hi res block, if into_or_from is set to 0
                # hig res block to low res block, if into_or_from is set to 1
                ed_remove.append(igrp[0])
                # number of new edges is equal to number of pixels on one side of the box (in 2D and D too)
                nnewed = np.power(nsplit, self.data.ndim - 1)
                muleidxs = nm.tile(igrp, nnewed)
                # copy the low-res edge multipletime
                newed = self.edges[muleidxs, :]
                neweddir = self.edge_dir[muleidxs]
                local_node_ids = sr_tab[self.edge_dir[igrp] + self.data.ndim * into_or_from,:].T.flatten()
                # first or second (the actual) node id is substitued by new node indexes
                newed[:, 1 - into_or_from] = local_node_ids \
                             + ndoffset
                if self._edge_weight_table is not None:
                    self.add_edges(newed, neweddir, self.edge_group[igrp], edge_low_or_high=1)
                else:
                    self.add_edges(newed, neweddir, self.edge_group[igrp], edge_low_or_high=None)
        return ed_remove

    def split_voxel(self, ndid):
        """

        :param ndid: int-like, low resolution voxel_id
        :param nsplit: int-like number
        :param tile_shape: this parameter will be used in future
        :return:
        """
        # TODO use tile_shape instead of nsplit
        # nsplit - was size of split square, tiles_shape = [nsplit, nsplit]
        # generate subgrid
        # tile_shape = tuple(tile_shape)
        # nsplit = tile_shape[0]
        # tile_shape = (nsplit, nsplit)
        nsplit = self.nsplit
        tile_shape = self._tile_shape
        if tile_shape in self.cache:
            nd, ed, ed_dir = self.cache[tile_shape]
        else:
            nd, ed, ed_dir = self.gen_grid_fcn(tile_shape, self.voxelsize / nsplit)
            # nd, ed, ed_dir = gen_base_graph(tile_shape, self.voxelsize / tile_shape)
            self.cache[tile_shape] = nd, ed, ed_dir

        ndoffset = self.lastnode
        # in new implementation nodes are 2D on 2D shape and 3D in 3D shape
        # in old implementation nodes are always 3D
        sr_tab = self.srt.get_sr_subtab(tile_shape)
        if self.compute_msindex:
            self.msi.set_block_higres(ndid, self.srt.inds + ndoffset)
        nd = make_nodes_3d(nd)
        self.add_nodes(nd + self.nodes[ndid,:] - (self.voxelsize3 / 2))
        if self._edge_weight_table is not None:
            # high resolution
            self.add_edges(ed + ndoffset, ed_dir, edge_low_or_high=1)
        else:
            self.add_edges(ed + ndoffset, ed_dir, edge_low_or_high=None)

        # connect subgrid
        ed_remove = []
        # sr_tab_old = self.sr_tab[nsplit]

        idxs = nm.where(self.edge_flag > 0)[0]

        # edges "into" node?
        ed_remove = self._edge_group_substitution( ndid, nsplit, idxs, sr_tab, ndoffset, ed_remove, into_or_from=0)

        # edges "from" node?
        ed_remove = self._edge_group_substitution( ndid, nsplit, idxs, sr_tab, ndoffset, ed_remove, into_or_from=1)


        # remove node
        self.node_flag[ndid] = False
        # remove edges
        self.edge_flag[ed_remove] = False

    def generate_base_grid(self, vtk_filename=None):
        """
        Run first step of algorithm. Next step is split_voxels
        :param vtk_filename:
        :return:
        """
        nd, ed, ed_dir = self.gen_grid_fcn(self.data.shape, self.voxelsize)
        self.add_nodes(nd)
        self.add_edges(ed, ed_dir, edge_low_or_high=0)

        if vtk_filename is not None:
            self.write_vtk(vtk_filename)

    def split_voxels(self, vtk_filename=None):
        """
        Second step of algorithm
        :return:
        """
        self.cache = {}
        self.msi = MultiscaleIndex(self.data.shape, block_size=self.nsplit)

        # old implementation
        # idxs = nm.where(self.data)
        # nr, nc = self.data.shape
        # for k, (ir, ic) in enumerate(zip(*idxs)):
        #     ndid = ic + ir * nc
        #     self.split_voxel(ndid, self.nsplit)

        # new_implementation
        # for ndid in np.flatnonzero(self.data):
        #     self.split_voxel(ndid, self.nsplit)

        # even newer implementation
        for ndid, val in enumerate(self.data.ravel()):
            if val == 0:
                if self.compute_msindex:
                    self.msi.set_block_lowres(ndid, ndid)
            else:
                self.split_voxel(ndid)

        self.finish()
        if vtk_filename is not None:
            self.write_vtk(vtk_filename)

    def run(self, base_grid_vtk_fn=None, final_grid_vtk_fn=None):
        # cache dict.
        self.cache = {}

        # generate base grid
        self.generate_base_grid(base_grid_vtk_fn)
        # self.generate_base_grid()
        # split voxels
        self.split_voxels(final_grid_vtk_fn)
        # self.split_voxels()


    def __init__(self, data, voxelsize, grid_function=None, nsplit=3, compute_msindex=True, edge_weight_table=None, compute_low_nodes_index=True):
        """

        :param data:
        :param voxelsize:
        :param grid_function: '2d' or 'nd'. Use '2d' for former implementation
        :param nsplit: size of low resolution block
        :param compute_msindex: compute indexes of nodes arranged in a ndarray with the same shape as higres image
        :param edge_weight_table: ndarray with size 2 * self.img.ndims. First axis describe whether is the edge
        between lowres(0) or highres(1) or hight-low (1) voxels. Second axis describe edge direction (edge axis).
        """
        # same dimension as data
        self.voxelsize = nm.asarray(voxelsize)
        # always 3D
        self.voxelsize3 = np.zeros([3])
        self.voxelsize3[:len(voxelsize)] = voxelsize

        self.data = np.asarray(data)
        if self.voxelsize.size != len(data.shape):
            logger.error("Datashape should be the same as voxelsize")
            raise ValueError("Datashape should be the same as voxelsize")
        self._edge_weight_table = edge_weight_table
        self.compute_low_node_inverse = compute_low_nodes_index
        # estimate maximum node number as number of lowres nodes + number of higres nodes + (nsplit - 1)^dim
        # 2d (nsplit, req) 2:3, 3:8, 4:12
        # 3D
        number_of_resized_nodes = np.count_nonzero(self.data)
        # self.ndmax_debug = data.size + number_of_resized_nodes* np.power(nsplit, self.data.ndim)
        self.ndmax = data.size + number_of_resized_nodes * np.power(nsplit, self.data.ndim)
        # init nodes
        self.nnodes = 0
        self.lastnode = 0
        self.nodes = nm.zeros((self.ndmax, 3), dtype=nm.float32)
        # node_flag: if true, this node is used in final output
        self.node_flag = nm.zeros((self.ndmax,), dtype=nm.bool)

        # init edges
        # estimate maximum number of dges as number of nodes multiplied by number of directions
        # the rest part is
        # if self.data.ndim == 2:
        #     edmax_rest = (9 * nsplit - 12)
        # elif self.data.ndim == 3:
        #     # edmax_rest = (9 * nsplit - 12) + 8 * nsplit**2 - 9 * nsplit - 35
        #     edmax_rest = 8 * nsplit**2 - 47
        # else:
        #     # this is not so efficient but should work
        #     edmax_rest = 9 * nsplit**(self.data.ndim - 1)
        # number of edges from every node
        edmax_from_node = self.data.ndim * self.ndmax
        # edges from lowres node to higres node from all directions
        edmax_into_node = number_of_resized_nodes * self.data.ndim * nsplit**(self.data.ndim - 1)
        self.edmax = edmax_from_node + edmax_into_node
        edmax = self.edmax
        # self.edmax_debug = edmax_from_node + edmax_into_node
        self.nedges = 0
        self.lastedge = 0
        eddtype = get_efficient_signed_int_type(self.ndmax)
        self.edges = - nm.ones((edmax, 2), dtype=eddtype)
        # edge_flag: if true, this edge is used in final output
        self.edge_flag = nm.zeros((edmax,), dtype=nm.bool)
        self.edge_dir = nm.zeros((edmax,), dtype=nm.int8)
        if self._edge_weight_table is not None:
            # dtype is given by graph-cut
            self.edges_weights = nm.zeros((edmax,), dtype=nm.int16)
        # list of edges on low resolution
        edgrdtype = get_efficient_signed_int_type(edmax)
        self.edge_group = - nm.ones((edmax,), dtype=edgrdtype)
        self.nsplit = nsplit
        self.compute_msindex = compute_msindex
        # indexes of nodes arranged in ndimage
        self.msinds = None
        if grid_function in (None, "nd", "ND"):
            self.gen_grid_fcn=gen_grid_nd
        elif grid_function in ("2d", "2D"):
            self.gen_grid_fcn=gen_grid_2d
        else:
            self.gen_grid_fcn=grid_function

        self._tile_shape = tuple(np.tile(nsplit, self.data.ndim))
        self.srt = SRTab()


def get_efficient_signed_int_type(number):
    if number < np.iinfo(np.int16).max:
        eddtype = np.int16
    elif number < np.iinfo(np.int32).max:
        eddtype = np.int32
    elif number < np.iinfo(np.int64).max:
        eddtype = np.int64
    elif number < np.iinfo(np.int128).max:
        eddtype = np.int128
    else:
        logger.error("Edge number higher than int128.")

    return eddtype


class SRTab(object):
    """
    Table of connection on transition between low resolution and high resolution
    """
    def __init__(self):
        self.sr_tab = {}
        # self.set_new_shape(shape)

    def set_new_shape(self, shape):
        self.shape = shape
        if len(shape) not in (2, 3):
            logger.error("2D or 3D shape expected")

        self.inds = np.array(range(np.prod(self.shape)))
        # direction_order = [0, 1, 2, 3, 4, 5, 6]
        reshaped = self.inds.reshape(self.shape)

        tab = []
        for direction in range(len(self.shape) - 1, -1, -1):
            # direction = direction_order[i]
            tab.append(reshaped.take(0, direction).flatten())
        for direction in range(len(self.shape) - 1, -1, -1):
            # direction = direction_order[i]
            tab.append(reshaped.take(-1, direction).flatten())
        self.sr_tab[tuple(shape)] = np.array(tab)


    def get_sr_subtab(self, shape):
        shape = tuple(shape)
        if shape not in self.sr_tab:
            self.set_new_shape(shape)
        return self.sr_tab[shape]


def grid_edges(shape, inds=None, return_directions=True):
    """
    Get list of grid edges
    :param shape:
    :param inds:
    :param return_directions:
    :return:
    """
    if inds is None:
        inds = np.arange(np.prod(shape)).reshape(shape)
    # if not self.segparams['use_boundary_penalties'] and \
    #         boundary_penalties_fcn is None :
    if len(shape) == 2:
        edgx = np.c_[inds[:, :-1].ravel(), inds[:, 1:].ravel()]
        edgy = np.c_[inds[:-1, :].ravel(), inds[1:, :].ravel()]

        edges = [ edgx, edgy]

        directions = [
            np.ones([edgx.shape[0]], dtype=np.int8) * 0,
            np.ones([edgy.shape[0]], dtype=np.int8) * 1,
            ]


    elif len(shape) == 3:
        # This is faster for some specific format
        edgx = np.c_[inds[:, :, :-1].ravel(), inds[:, :, 1:].ravel()]
        edgy = np.c_[inds[:, :-1, :].ravel(), inds[:, 1:, :].ravel()]
        edgz = np.c_[inds[:-1, :, :].ravel(), inds[1:, :, :].ravel()]
        edges = [ edgx, edgy, edgz]
    else:
        logger.error("Expected 2D or 3D data")

    # for all edges along first direction put 0, for second direction put 1, for third direction put 3
    if return_directions:
        directions = []
        for idirection in range(len(shape)):
            directions.append(
                np.ones([edges[idirection].shape[0]], dtype=np.int8) * idirection
            )
    edges = np.concatenate(edges)
    if return_directions:
        edge_dir = np.concatenate(directions)
        return edges, edge_dir
    else:
        return edges

def grid_nodes(shape, voxelsize=None):
    voxelsize = np.asarray(voxelsize) # [:len(shape)]
    nodes = np.moveaxis(np.indices(shape), 0, -1).reshape(-1, len(shape))
    if voxelsize is not None:
        nodes = (nodes * voxelsize) + (0.5 * voxelsize)
    return nodes

def gen_grid_nd(shape, voxelsize=None, inds=None):
    edges, edge_dir = grid_edges(shape, inds, return_directions=True)
    # nodes coordinates
    nodes = grid_nodes(shape, voxelsize)
    return nodes, edges, edge_dir

def gen_grid_2d(shape, voxelsize):
    """
    Generate list of edges for a base grid.
    """
    nr, nc = shape
    nrm1, ncm1 = nr - 1, nc - 1
    # sh = nm.asarray(shape)
    # calculate number of edges, in 2D: (nrows * (ncols - 1)) + ((nrows - 1) * ncols)
    nedges = 0
    for direction in range(len(shape)):
        sh = copy.copy(list(shape))
        sh[direction] += -1
        nedges += nm.prod(sh)


    nedges_old = ncm1 * nr + nrm1 * nc
    edges = nm.zeros((nedges, 2), dtype=nm.int16)
    edge_dir = nm.zeros((ncm1 * nr + nrm1 * nc, ), dtype=nm.bool)
    nodes = nm.zeros((nm.prod(shape), 3), dtype=nm.float32)

    # edges
    idx = 0
    row = nm.zeros((ncm1, 2), dtype=nm.int16)
    row[:,0] = nm.arange(ncm1)
    row[:,1] = nm.arange(ncm1) + 1
    for ii in range(nr):
        edges[slice(idx, idx + ncm1),:] = row + nc * ii
        idx += ncm1

    edge_dir[slice(0, idx)] = 0 # horizontal dir

    idx0 = idx
    col = nm.zeros((nrm1, 2), dtype=nm.int16)
    col[:,0] = nm.arange(nrm1) * nc
    col[:,1] = nm.arange(nrm1) * nc + nc
    for ii in range(nc):
        edges[slice(idx, idx + nrm1),:] = col + ii
        idx += nrm1

    edge_dir[slice(idx0, idx)] = 1 # vertical dir

    # nodes
    idx = 0
    row = nm.zeros((nc, 3), dtype=nm.float32)
    row[:,0] = voxelsize[0] * (nm.arange(nc) + 0.5)
    row[:,1] = voxelsize[1] * 0.5
    for ii in range(nr):
        nodes[slice(idx, idx + nc),:] = row
        row[:,1] += voxelsize[1]
        idx += nc

    return nodes, edges, edge_dir

def make_nodes_3d(nodes):
    if nodes.shape[1] == 2:
        zeros = np.zeros([nodes.shape[0], 1], dtype=nodes.dtype)
        nodes = np.concatenate([nodes, zeros], axis=1)
    return nodes

def write_grid_to_vtk(fname, nodes, edges, node_flag=None, edge_flag=None):
    """
    Write nodes and edges to VTK file
    :param fname: VTK filename
    :param nodes:
    :param edges:
    :param node_flag: set if this node is really used in output
    :param edge_flag: set if this flag is used in output
    :return:
    """

    if node_flag is None:
        node_flag = np.ones([nodes.shape[0]], dtype=np.bool)
    if edge_flag is None:
        edge_flag = np.ones([edges.shape[0]], dtype=np.bool)
    nodes = make_nodes_3d(nodes)
    f = open(fname, 'w')

    f.write('# vtk DataFile Version 2.6\n')
    f.write('output file\nASCII\nDATASET UNSTRUCTURED_GRID\n')

    idxs = nm.where(node_flag > 0)[0]
    nnd = len(idxs)
    aux = -nm.ones(node_flag.shape, dtype=nm.int32)
    aux[idxs] = nm.arange(nnd, dtype=nm.int32)
    f.write('\nPOINTS %d float\n' % nnd)
    for ndi in idxs:
        f.write('%.6f %.6f %.6f\n' % tuple(nodes[ndi,:]))

    idxs = nm.where(edge_flag > 0)[0]
    ned = len(idxs)
    f.write('\nCELLS %d %d\n' % (ned, ned * 3))
    for edi in idxs:
        f.write('2 %d %d\n' % tuple(aux[edges[edi,:]]))

    f.write('\nCELL_TYPES %d\n' % ned)
    for edi in idxs:
        f.write('3\n')

class MultiscaleIndex(object):
    def __init__(self, shape, block_size):
        self.shape = np.asarray(shape)
        self.msinds = np.zeros(self.shape * block_size, dtype=int)
        self.block_size = block_size
        self.block_shape = [block_size] * self.msinds.ndim
        self.cache_slice = [None] * self.msinds.ndim

    def _prepare_cache_slice(self, index):
        coords = np.unravel_index(index, self.shape)

        for ax, single_ax_coord in enumerate(coords):
            coord_higres_start = single_ax_coord * self.block_size
            self.cache_slice[ax] = slice(coord_higres_start, coord_higres_start + self.block_size)

    def set_block_lowres(self, index, val):
        self._prepare_cache_slice(index)
        self.msinds[self.cache_slice] = val

    def set_block_higres(self, index, val):
        self._prepare_cache_slice(index)
        self.msinds[self.cache_slice] = np.asarray(val).reshape(self.block_shape)


# def relabel(arr, forward_indexes=None):
#     # for x in np.nditer(arr, op_flags=["readwrite"]):
#     #     x[...] = forward_indexes[x]
#     pass
